/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam;

import rubeam.model.Element;
import rubeam.model.Node;
import rubeam.model.Structure;
import rubeam.model.loads.ElementConLoadAng;
import rubeam.model.loads.ElementMoment;
import rubeam.model.loads.ElementUniDistribLoadNor;
import rubeam.model.loads.ElementUniDistribLoadSecDir;
import rubeam.model.loads.ElementUniDistribLoadSecNor;

public class Application {

    public static void main(String[] args) {

        // nodes
        Node a = new Node(0, 0, "a");
        Node b = new Node(4, 0, "b");
        Node c = new Node(0, 2, "c");
        Node d = new Node(4, 2, "d");
        Node e = new Node(0, 4, "e");
        Node f = new Node(2, 5, "f");
        Node g = new Node(4, 4, "g");
        // beams
        Element b1 = new Element(a, c, "1");
        Element b2 = new Element(b, d, "2");
        Element b3 = new Element(c, d, "3");
        Element b4 = new Element(c, e, "4");
        Element b5 = new Element(e, f, "5");
        Element b6 = new Element(f, g, "6");
        Element b7 = new Element(g, d, "7");

        // structure
        Structure struct = new Structure();
        struct.addElement(b1);
        struct.addElement(b2);
        struct.addElement(b3);
        struct.addElement(b4);
        struct.addElement(b5);
        struct.addElement(b6);
        struct.addElement(b7);

        // loads
        ElementUniDistribLoadSecNor l1 = new ElementUniDistribLoadSecNor(10000,
                0.1, 1.8, "li");
        struct.addElementLoad(l1);
        struct.putElementLoad(b1, l1);
        
        ElementUniDistribLoadSecDir l2 = new ElementUniDistribLoadSecDir(10000,
                0.3, 1.5, "l2");
        struct.addElementLoad(l2);
        struct.putElementLoad(b2, l2);

        ElementUniDistribLoadNor l3 = new ElementUniDistribLoadNor(10000, "l3");
        struct.addElementLoad(l3);
        struct.putElementLoad(b3, l3);

        ElementMoment l4 = new ElementMoment(10000, 0.6, "l4");
        struct.addElementLoad(l4);
        struct.putElementLoad(b4, l4);
        
        ElementUniDistribLoadNor l5 = new ElementUniDistribLoadNor(10000, "l5");
        struct.addElementLoad(l5);
        struct.putElementLoad(b5, l5);

        double angle6 = Math.toRadians(-90);
        ElementConLoadAng l6 = new ElementConLoadAng(10000, angle6, 1.0, "l6");
        struct.addElementLoad(l6);
        struct.putElementLoad(b6, l6);

        double angle7 = Math.toRadians(0);
        ElementConLoadAng l7 = new ElementConLoadAng(10000, angle7, 0.3, "l7");
        struct.addElementLoad(l7);
        struct.putElementLoad(b7, l7);

        // constraints
        a.setContraints(0, 0, 0);
        b.setContraints(0, 0, 0);
        // f.setContraints(Double.NaN, .0134, Double.NaN);

        // truss system element3!
        // b3.setCi(false);
        // b3.setCk(false);

        // truss system on top!
        // b5.setCi(false);
        // b5.setCk(false);
        // b6.setCi(false);
        // b6.setCk(false);

        // set inter steps
        struct.setResultSteps(2);

        // solve system
        // struct.solve();
        // Matrix resultVg = struct.getResultVg();

        // print results
        // System.out.println("globV = 1e-4 *");
        // resultVg.times(1e3).print(10, 4);
        System.out.println("globVa = ");
        struct.getNodeResultV(a).times(1).print(3, 10);
        System.out.println("globVb = ");
        struct.getNodeResultV(b).times(1).print(3, 10);
        System.out.println("globVc = ");
        struct.getNodeResultV(c).times(1).print(3, 10);
        System.out.println("globVd = ");
        struct.getNodeResultV(d).times(1).print(3, 10);
        System.out.println("globVe = ");
        struct.getNodeResultV(e).times(1).print(3, 10);
        System.out.println("globVf = ");
        struct.getNodeResultV(f).times(1).print(3, 10);
        System.out.println("globVg = ");
        struct.getNodeResultV(g).times(1).print(3, 10);

    }
}
