/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import rubeam.engine.ParseException;
import rubeam.engine.RuBeamEngine;
import rubeam.engine.TokenMgrError;

public class Engine {

    private static final String SWITCH_IMPORTFILE = "-f";
    
    public static void main(String[] args) {

        // Standard
        if (args.length == 0) {
            RuBeamEngine engine = new RuBeamEngine(System.in);
            while (true)
            {
              engine.ReInit(System.in);
              System.out.print("ruBeam> ");
              System.out.flush();
              try
              {
                engine.start();
              }
              catch (TokenMgrError e)
              {
                System.err.println("Error: " + e);
                System.out.print("ruBeam> ");
                System.out.flush();
              }
              catch (ParseException e)
              {
                System.err.println("Exception: " + e);
                System.out.print("ruBeam> ");
                System.out.flush();
              }
            }
        }
        // Run with arguments
        else {
            File file = null;
            for (int i = 0; i < args.length; i++) {
                if (args[i].equalsIgnoreCase(SWITCH_IMPORTFILE)) {
                    file = new File(args[i+1]);
                    break;
                }
            }
            if (file != null) {
                if (! file.exists())
                    System.err.println("File does not exist!");
                else if (! file.canRead())
                    System.err.println("Cannot read from file!");
                else {
                    try {
                        RuBeamEngine engine = new RuBeamEngine(new FileInputStream(file));
                        engine.setPromptEnabled(false);
                        engine.start();
                        engine.setPromptEnabled(true);
                    } catch (FileNotFoundException e) {
                        System.err.println(e.getMessage());
                    } catch (ParseException e) {
                        System.err.println(e.getMessage());
                    }
                }
            }
        }
        
    }
}
