/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.model;

import java.util.ArrayList;
import java.util.List;

import Jama.Matrix;

public class EquationSystem {

    public static final boolean dump = false;

    public static Matrix solve(Matrix[] globals) {
        Matrix globK = globals[0];
        Matrix globS = globals[1];
        Matrix globV = globals[2];

        int dim = globV.getRowDimension();

        List<Integer> nonZeros = new ArrayList<Integer>();
        for (int i = 0; i < dim; i++) {
            if (Double.isNaN(globV.get(i, 0)) && globK.get(i, i) != 0 ) {
                nonZeros.add(i);
            }
            if ( Double.isNaN(globV.get(i, 0)) ) {
                globV.set(i, 0, 0);
            }
        }
        for (int i = 0; i < dim; i++) {      
            if ( globV.get(i, 0) != 0 ) {                
                globS.plusEquals(globK.getMatrix(0, dim-1, i, i).times(globV.get(i, 0)));
            }  
        }

        int[] ids = new int[nonZeros.size()];
        for (int i = 0; i < nonZeros.size(); i++) {
            ids[i] = nonZeros.get(i); 
        }
        // check if system is directly solvable
        Matrix resultVg = globV;
        if (!nonZeros.isEmpty()){
            Matrix globKr = globK.getMatrix(ids, ids);
            // check if system is kinematic
            if (globKr.getRowDimension() > globKr.rank()) {
                throw new IllegalStateException("System is kinematic!");
            }
            Matrix globSr = globS.getMatrix(ids, new int[] { 0 });
            Matrix resultVr = globKr.inverse().times(globSr.uminus());
            resultVg.setMatrix(ids, new int[] { 0 }, resultVr);
        }
        if (dump) {
            Matrix resultVgP = resultVg.times(1e3);
            System.out.println("resultVg = 1e-3 * ");
            resultVgP.print(8, 6);
        }
        return resultVg;
    }
}
