/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.model;

import java.io.Serializable;

public abstract class ModelObject implements Serializable, Cloneable {

    private static final long serialVersionUID = 1L;
    protected String name; 
    private IChangeListener listener = null;
    
    protected ModelObject(String name) {
        this.name = name;
    }
    
    public String getName() {
        return name;
    }
    public void setName(String name) {
        this.name = name;
    }
    
    public void setChangeListener(IChangeListener listener) {
        this.listener = listener;
    }
    
    protected void _fireWasChanged() {
        if (listener != null)
            listener.wasChanged();
    }
    
    public Object clone() {
        try {
            return super.clone(); // Makes a shallow copy
        } catch (CloneNotSupportedException e) {
            // This shouldn't happen, since we are Cloneable
            throw new InternalError();
        }
    }
}
