/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.model.loads;

import rubeam.Util;
import rubeam.model.Element;
import rubeam.model.ModelObject;
import rubeam.model.Structure;
import Jama.Matrix;

public abstract class AbstractElementLoad extends ModelObject  {

    private static final long serialVersionUID = 1L;
    
    protected AbstractElementLoad(String name) {
        super(name);
    }
   
    public Element getElement() {
        Structure struct = Structure.getStructure();
        Element element = struct.getAssignedElement(this);
        return element;
    }
    
    // AbstractElementLoad interface
    public abstract Matrix[] getLoadVectors(double xs);

    // helpers
    protected Matrix[] _buildLoadVectors(Matrix Lji) {

        // [L1, L2, Lt]
        Matrix[] Ls = new Matrix[3];
        Matrix L1 = new Matrix(3, 1);
        Matrix L2 = new Matrix(3, 1);
        Ls[0] = L1;
        Ls[1] = L2;

        Element element = getElement();
        Matrix D = Util.getRotMatrix(element.getNodeI(), element.getNodeK());

        Matrix L = D.times(Lji);

        Ls[2] = L;

        // cases (fixed, joined)
        // 1/ 2
        if ((!element.getHingeI() && !element.getHingeK()) || (element.getHingeI() && !element.getHingeK())) {

            L1.set(0, 0, L.get(2, 0));
            L1.set(1, 0, -L.get(1, 0));
            L1.set(2, 0, -L.get(0, 0));

            L2.set(0, 0, -L.get(3, 0));
            L2.set(1, 0, -L.get(4, 0));
            L2.set(2, 0, -L.get(5, 0));

        }
        // 3/ 4
        else if ((!element.getHingeI() && element.getHingeK())
                || (element.getHingeI() && element.getHingeK())) {

            L1.set(0, 0, -L.get(3, 0));
            L1.set(1, 0, -L.get(1, 0));
            L1.set(2, 0, -L.get(0, 0));

            L2.set(0, 0, 0);
            L2.set(1, 0, -L.get(4, 0));
            L2.set(2, 0, -L.get(5, 0));

        }
        else {
            throw new IllegalStateException();
        }

        return Ls;
    }
}
