/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.model.loads;

import rubeam.Util;
import rubeam.model.Element;
import Jama.Matrix;

public class ElementLeapNor extends AbstractElementLoad {

    private static final long serialVersionUID = 1L;

    private double wE;
    private double li;

    public ElementLeapNor( double wE, double li, String name) {
        super(name);
        this.wE = wE;
        this.li = li;
    }

    public double getWE() {
        return this.wE;
    }
    public void setWE(double wE) {
        this.wE = wE;
        _fireWasChanged();
    }

    public double getLi() {
        return this.li;
    }
    public void setLi(double li) {
        this.li = li;
        _fireWasChanged();
    }

    public Matrix[] getLoadVectors(double xs) {

        if (xs >= li) {
            Element element = getElement();
            Matrix Bjs = element.getBj(xs - li);
            double EI = element.getSection().getEI();
            double NII = element.getNII();

            double[][] _Lji = new double[6][1];
            _Lji[0][0] = 0; // ujL
            _Lji[1][0] = Bjs.get(0, 0) * wE; // wjR
            _Lji[2][0] = Bjs.get(0, 1) * NII / EI * wE; // phijR
            _Lji[3][0] = -Bjs.get(0, 0) * NII * wE; // MjR
            _Lji[4][0] = 0; // RjR
            _Lji[5][0] = 0; // NjL
            Matrix Lji = new Matrix(_Lji);
            return _buildLoadVectors(Lji);
        }
        else {
            Matrix Lji = new Matrix(6, 1, 0.0);
            return _buildLoadVectors(Lji);
        }
    }

    public String toString() {
        return "PointLoad wE=" + Util.format(wE);
    }
}
