/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.model.loads;

import rubeam.model.Element;
import Jama.Matrix;

public class ElementSecOrder extends AbstractElementLoad {

    private static final long serialVersionUID = 1L;
    
    public ElementSecOrder(String name) {
        super(name);
    }
    public Matrix[] getLoadVectors(double xs) {
        Element element = getElement();

        Matrix Bj = element.getBj(xs);
        double EI = element.getSection().getEI();
        double NII = element.getNII();
        double q = - (NII * 8 * element.getW0() / (xs * xs));
        double phiVi = element.getPsi0() + 4 * element.getW0() / element.getLength();

        double[][] _Lji = new double[6][1];
        _Lji[0][0] = 0; // ujL
        _Lji[1][0] = Bj.get(0, 4) / EI * q + Bj.get(0, 3) / EI * NII * phiVi; // wjR
        _Lji[2][0] = Bj.get(0, 3) / EI * q + Bj.get(0, 2) / EI * NII * phiVi; // phijR
        _Lji[3][0] = -Bj.get(0, 2) * q - Bj.get(0, 1) * NII * phiVi; // MjR
        _Lji[4][0] = 0; // RjR
        _Lji[5][0] = 0; // NjL
        Matrix Lji = new Matrix(_Lji);

        return _buildLoadVectors(Lji);
    }
}
