/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.model.loads;

import rubeam.Util;
import rubeam.model.Element;
import Jama.Matrix;

public class ElementUniDistribMomentSec extends AbstractElementLoad {

    private static final long serialVersionUID = 1L;

    private double dm;
    private double li;
    private double lj;

    public ElementUniDistribMomentSec(double dm, double li, double lj, String name) {
        super(name);
        this.dm = dm;
        this.li = li;
        this.lj = lj;
    }

    public double getDm() {
        return this.dm;
    }
    public void setDm(double dm) {
        this.dm = dm;
        _fireWasChanged();
    }

    public double getLi() {
        return this.li;
    }
    public void setLi(double li) {
        this.li = li;
        _fireWasChanged();
    }
    
    public double getLj() {
        return this.lj;
    }
    public void setLj(double lj) {
        this.lj = lj;
        _fireWasChanged();
    }

    public Matrix[] getLoadVectors(double xs) {
        if (xs <= li) {
            Matrix Lji = new Matrix(6, 1, 0.0);
            return _buildLoadVectors(Lji);
        }
        else {
            Element element = getElement();
            Matrix Bjs = element.getBj(xs - li);
            double EI = element.getSection().getEI();

            double[][] _Lji = new double[6][1];
            _Lji[0][0] = 0; // ujL
            _Lji[1][0] = -Bjs.get(0, 3) / EI * dm; // wjR
            _Lji[2][0] = -Bjs.get(0, 2) / EI * dm; // phijR
            _Lji[3][0] = Bjs.get(0, 1) * dm; // MjR
            _Lji[4][0] = 0; // QjR
            _Lji[5][0] = 0; // NjL

            if (xs > lj) {
                Matrix Bjss = element.getBj(xs - lj);

                _Lji[0][0] = 0; // ujL
                _Lji[1][0] = _Lji[1][0] + (Bjss.get(0, 3) / EI * dm); // wjR
                _Lji[2][0] = _Lji[2][0] + (Bjss.get(0, 2) / EI * dm); // phijR
                _Lji[3][0] = _Lji[3][0] - Bjss.get(0, 1) * dm; // MjR
                _Lji[4][0] = 0; // RjR
                _Lji[5][0] = 0; // NjL
            }
            Matrix Lji = new Matrix(_Lji);
            return _buildLoadVectors(Lji);
        }
    }

    public String toString() {
        return "UniDistribLoad dm=" + Util.format(dm);
    }
}
