/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.model.loads;

import rubeam.Util;
import Jama.Matrix;

public class NodalLoadAng extends AbstractNodeLoad {

    private static final long serialVersionUID = 1L;

    private double P;
    private double angle;

    public NodalLoadAng(double P, double angle, String name) {
        super(name);
        this.P = P;
        this.angle = angle;
    }

    public void setP(double value) {
        this.P = value;
        _fireWasChanged();
    }
    public double getP() {
        return this.P;
    }
    public void setAngle(double angle) {
        this.angle = angle;
        _fireWasChanged();
    }
    public double getAngle() {
        return this.angle;
    }

    public Matrix getLoadMatrix() {
        // Mi, Vi, Hi
        double v = -P * Math.sin(angle);
        double h = P * Math.cos(angle);
        Matrix m = new Matrix(3, 1, 0);
        m.set(1, 0, v);
        m.set(2, 0, h);
        return m;
    }

    public String toString() {
        return "NodalLoad '"+ this.name + "' P=" + Util.format(P) 
            + ", \u03b1=" + Util.format(Math.toDegrees(angle));
    }
}
