/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.model.sections;

import rubeam.model.ModelObject;

public abstract class AbstractSection extends ModelObject {
    
    private static final long serialVersionUID = 1L;
    private final static AbstractSection defaultSection = new DefaultSection(
            .3 * .1, .3 * .1, (.3 * .3 * .3 * .1 / 12), 2e+11, "Default");
    protected double E;
    
    protected AbstractSection(String name, double E) {
        super(name);
        this.E = E;
    }
    
    // Standard section
    public static AbstractSection getDefaultSection() {
        return defaultSection;
    }
    
    public double getE() {
        return E;
    }
    public void setE(double E) {
        this.E = E;
        _fireWasChanged();
    }
    public double getEI() {
        return this.E * getIi();
    }
    
    // Section interface
    public abstract double getHi();
    public abstract double getHk();
    public abstract double getS();
    public abstract double getAi();
    public abstract double getAk();
    public abstract double getIi();
    
    // empty set methods, need to override
    public double getB() {
        throw new UnsupportedOperationException("Can't get b!");
    }
    public double getT() {
        throw new UnsupportedOperationException("Can't get t!");
    }
    public void setHi(double hi){
        throw new UnsupportedOperationException("Can't set hi!");
    };
    public void setHk(double hk){
        throw new UnsupportedOperationException("Can't set hk!");
    };
    public void setAi(double Ai){
        throw new UnsupportedOperationException("Can't set Ai!");
    };
    public void setAk(double Ak){
        throw new UnsupportedOperationException("Can't set Ak!");
    };
    public void setIi(double Ii){
        throw new UnsupportedOperationException("Can't set Ii!");
    };
    public void setB(double b){
        throw new UnsupportedOperationException("Can't set b!");
    };
    public void setS(double s){
        throw new UnsupportedOperationException("Can't set s!");
    };
    public void setT(double t){
        throw new UnsupportedOperationException("Can't set t!");
    };    
}
