/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds;

import rubeam.RuBeam;
import rubeam.gui.RuBeamMenu;
import cib.cad.kernel.Kernel;
import cib.util.cmd.Cmd;
import cib.util.cmd.CmdAbortedException;

public class AutoAnalyse implements Cmd {

    private boolean m_auto;
    private boolean m_changed = false;
    
    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        m_auto = RuBeam.isAutoAnalysing();
        
        String auto = krnl.readString("Analyse automatically? (on/off)");
        if (auto.equalsIgnoreCase("on")) {
            if (m_auto) {
                m_changed = false;
                return;
            }
            RuBeam.setAutoAnalysing(! m_auto);
            RuBeamMenu.s_cbItem.setSelected(! m_auto);
            m_changed = true;
        }
        else {
            if (! m_auto) {
                m_changed = false;
                return;
            }
            RuBeam.setAutoAnalysing(! m_auto);
            RuBeamMenu.s_cbItem.setSelected(! m_auto);
            m_changed = true;
        }
        
        System.out.println(this);
    }
    public void undoCmd(Object context) {
        if (m_changed) {
            RuBeam.setAutoAnalysing(m_auto);
            RuBeamMenu.s_cbItem.setSelected(m_auto);
        }
    }
    public void redoCmd(Object context) {
        if (m_changed) {
            RuBeam.setAutoAnalysing(! m_auto);
            RuBeamMenu.s_cbItem.setSelected(! m_auto);
        }
    }
    public boolean changesState() {
        return m_changed;
    }
    public boolean isUndoable() {
        return true;
    }
    public String toString() {
        return "Set auto-analysing to '" + ! m_auto + "'.";
    }
}
