/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds;

import java.util.HashSet;
import java.util.Set;

import rubeam.RuBeam;
import cib.cad.db.comp.Component;
import cib.cad.kernel.Kernel;
import cib.util.cmd.CmdAbortedException;
import cib.util.cmd.CmdAdapter;

public class ClearStructure extends CmdAdapter {
    private boolean m_ok = false;
    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        m_ok = krnl.readYesNo(
                "Clear structure?", 
                "Clear structure?", 
                "Do you really want to clear the structure? This command cannot be undone!", 
                Kernel.MSG_WARNING);
        if (m_ok) {
            // CADEMIA
            Set<Component> cmpSet = krnl.getDatabase().getComponentSet();
            Set<Component> remSet = new HashSet<Component>();
            for (Component comp : cmpSet) {
                if (comp.getClass().getPackage().getName().equals("rubeam.cmps"))
                    remSet.add(comp);
            }
            cmpSet.removeAll(remSet);
            
            // ruBeam
            RuBeam.getStructure().clear();
            System.out.println(this);
        }
    }
    public boolean changesState() {
        return m_ok;
    }
    public String toString() {
        return "Structure cleared.";
    }
}
