/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import rubeam.RuBeam;
import cib.cad.kernel.Kernel;
import cib.cad.kernel.interpreter.CmdLineInterpreter;
import cib.cad.kernel.interpreter.ParseException;
import cib.cad.kernel.interpreter.TokenMgrError;
import cib.util.cmd.CmdAbortedException;
import cib.util.cmd.CmdAdapter;

/** 
 * Runs a ruBeam macro file.
 * 
 */
public class ImportMacro extends CmdAdapter {
    
    public static final String MACRO_EXTENSION = "rb";
    
    public void doCmd(Object context) throws CmdAbortedException {
        final Kernel krnl = (Kernel)context;
        
        // Show file dialog
        File file = krnl.readFile(new Kernel.FileDialogConfiguration() {
            public boolean accept(File f) {
                if (f.isDirectory())
                    return true;
                String fileName = f.getName();
                if (! fileName.endsWith("." + MACRO_EXTENSION))
                    return false;
                if (! f.exists()) {
                    System.err.println(f.getAbsolutePath() + " doesn't exist");
                    return false;
                }
                if (! f.canRead()) {
                    System.err.println("Can't read from " +
                        f.getAbsolutePath());
                    return false;
                }
                return true;
            }
            public String[] getExtensions() {
                return new String[] {MACRO_EXTENSION};
            }
            public String getPrompt() {
                return "Import ruBeam macro";
            }
            public String getFileFormatDescription() {
                return "ruBeam-files (*." + MACRO_EXTENSION + ")";
            }
        });
        
        try {
            CmdLineInterpreter interpreter = new CmdLineInterpreter(System.in);

            interpreter.addListener(new CmdLineInterpreter.Listener() {
                private String m_cmdName = null;
                private String m_args = "";
                public void parsingCommand(String cmdName) {
                    m_cmdName = cmdName;
                    m_args = " ";
                }
                public void parsingArgument(String arg) {
                    m_args += arg + " ";
                }
                public void parsingEndOfCommand() {
                    krnl.doCmd(m_cmdName, m_args);
                }
            });
            
            interpreter.ReInit(new FileInputStream(file));
            
            try {
                RuBeam.setAutoUpdating(false);
                interpreter.start();
            }
            catch(TokenMgrError e) {
                System.err.println(e);
            }
            catch(ParseException e) {
                System.err.println(e);
            }
            finally {
                RuBeam.setAutoUpdating(true);
            }
            
        }
        catch(FileNotFoundException e) {
            System.err.println("Error: Macro file " + file + " not found");
        }
        catch(Exception e) {
            System.err.println("Macro error: ");
            e.printStackTrace();
        }
    }
}
