/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;

import rubeam.RuBeam;
import rubeam.gui.RuBeamMenu;
import cib.cad.Instance;
import cib.cad.kernel.Kernel;
import cib.util.RuntimeExtensionClassLoader;
import cib.util.cmd.CmdAbortedException;
import cib.util.cmd.CmdAdapter;

public class RegisterPlugin extends CmdAdapter {

    public void doCmd(Object context) throws CmdAbortedException {

        Kernel kernel = (Kernel) context;

        // Activate when building plugin!
        RuntimeExtensionClassLoader cl = kernel.getClassLoader();
        URL url;
        try {
            url = kernel.getPluginURL("PLUGIN_RUBEAM");
            if (url != null) {
                File file = new File(url.toURI());
                String path = file.getParentFile().getAbsolutePath()
                + "/Jama-1.0.2.jar";
                URL entryUrl = new URL(url.getProtocol() + ":" + path);
                cl.addURL(entryUrl);
            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (URISyntaxException e) {
            e.printStackTrace();
        }

        // First check CADEMIA version
        int version_major = Integer.parseInt(Instance.VERSION_MAJOR);
        int version_minor = Integer.parseInt(Instance.VERSION_MINOR);
        int idx = Instance.VERSION_BUILD.indexOf('-');
        int version_build = 0;
        if (idx < 0) {
            version_build = Integer.parseInt(Instance.VERSION_BUILD);
        }
        else {
            version_build = Integer.parseInt(Instance.VERSION_BUILD.substring(0, idx));
        }
        if (version_major < 1) {
            throw new IllegalStateException(
                    "$$$$$ Plugin not loaded. CADEMIA version " + version_major
                    + "." + version_minor + Instance.VERSION_ROBUSTNESS
                    + version_build
                    + " is to old. Use version 1.5b5 or newer. $$$$$");
        }
        else if (version_major == 1) {
            if (version_minor > 5) {
            }
            else if (version_minor < 5
                    || (version_minor == 5 && version_build <= 5)) {
                throw new IllegalStateException(
                        "$$$$$ Plugin not loaded. CADEMIA version ("
                        + version_major
                        + "."
                        + version_minor
                        + Instance.VERSION_ROBUSTNESS
                        + version_build
                        + ") is to old. Use version 1.5b5 or newer. $$$$$");
            }
        }

        // ruBeam base commands
        // components
        kernel.addCmd("addnode", rubeam.cmds.node.AddNode.class);
        kernel.addCmd("addelement", rubeam.cmds.elm.AddElement.class);
        kernel.addCmd("addgensection", rubeam.cmds.sec.AddLinearGeneralSection.class);
        kernel.addCmd("addrecsection", rubeam.cmds.sec.AddLinearRectangleSection.class);
        kernel.addCmd("addiprofsection", rubeam.cmds.sec.AddLinearIProfileSection.class);
        kernel.addCmd("addsandsection", rubeam.cmds.sec.AddLinearSandwichSection.class);
        // loads
        kernel.addCmd("addnodalloadang", rubeam.cmds.nodeload.AddNodalLoadAng.class);
        kernel.addCmd("addelementconload", rubeam.cmds.elmload.AddElementConLoadAng.class);
        kernel.addCmd("addelementunidisloadnor", rubeam.cmds.elmload.AddElementUniDistribLoadNor.class);
        // node handling
        kernel.addCmd("setnodename", rubeam.cmds.node.SetNodeName.class);
        kernel.addCmd("setnodex", rubeam.cmds.node.SetNodeX.class);
        kernel.addCmd("setnodey", rubeam.cmds.node.SetNodeY.class);
        kernel.addCmd("setnodeconstx", rubeam.cmds.node.SetNodeConstraintX.class);
        kernel.addCmd("setnodeconsty", rubeam.cmds.node.SetNodeConstraintY.class);
        kernel.addCmd("setnodeconstphi", rubeam.cmds.node.SetNodeConstraintPhi.class);
        kernel.addCmd("setnodeconsts", rubeam.cmds.node.SetNodeConstraints.class);
        kernel.addCmd("setnodefixed", rubeam.cmds.node.SetFixed.class);
        // element handling
        kernel.addCmd("setelementname", rubeam.cmds.elm.SetElementName.class);
        kernel.addCmd("setelementnodei", rubeam.cmds.elm.SetElementNodeI.class);
        kernel.addCmd("setelementnodek", rubeam.cmds.elm.SetElementNodeK.class);
        kernel.addCmd("assignsection", rubeam.cmds.sec.AssignSection.class);
        kernel.addCmd("setelementhingei", rubeam.cmds.elm.SetElementHingeI.class);
        kernel.addCmd("setelementhingek", rubeam.cmds.elm.SetElementHingeK.class);
        // section handling
        kernel.addCmd("setsectionname", rubeam.cmds.sec.SetSectionName.class);
        kernel.addCmd("setsectionai", rubeam.cmds.sec.SetSectionAi.class);
        kernel.addCmd("setsectionak", rubeam.cmds.sec.SetSectionAk.class);
        kernel.addCmd("setsectionb", rubeam.cmds.sec.SetSectionB.class);
        kernel.addCmd("setsectione", rubeam.cmds.sec.SetSectionE.class);
        kernel.addCmd("setsectionhi", rubeam.cmds.sec.SetSectionHi.class);
        kernel.addCmd("setsectionhk", rubeam.cmds.sec.SetSectionHk.class);
        kernel.addCmd("setsectionii", rubeam.cmds.sec.SetSectionIi.class);
        kernel.addCmd("setsections", rubeam.cmds.sec.SetSectionS.class);
        kernel.addCmd("setsectiont", rubeam.cmds.sec.SetSectionT.class);
        // sys commands
        kernel.addCmd("show", rubeam.cmds.DumpStructure.class);
        kernel.addCmd("analyse", rubeam.cmds.Analyse.class);
        kernel.addCmd("setvisualizesteps", rubeam.cmds.SetVisualizeSteps.class);
        kernel.addCmd("setresultsteps", rubeam.cmds.SetResultSteps.class);
        kernel.addCmd("autoanalyse", rubeam.cmds.AutoAnalyse.class);
        
        //
        // Plugin Commands
        // node handling
        kernel.addCmd("removenode", rubeam.cmds.node.RemoveNode.class);
        // nodal load handling
        kernel.addCmd("assignnodalload", rubeam.cmds.nodeload.AssignNodeLoads.class);
        kernel.addCmd("assignnodalloads", rubeam.cmds.nodeload.AssignNodeLoads.class);
        kernel.addCmd("removenodalload", rubeam.cmds.nodeload.RemoveNodeLoad.class);
        kernel.addCmd("setnodalloadvalue", rubeam.cmds.nodeload.SetNodeLoadValue.class);
        kernel.addCmd("setnodalloadangle", rubeam.cmds.nodeload.SetNodeLoadAngle.class);
        kernel.addCmd("setnodalloadname", rubeam.cmds.nodeload.SetNodeLoadName.class);
        // element handling
        kernel.addCmd("removeelement", rubeam.cmds.elm.RemoveElement.class);
        // element load handling
        kernel.addCmd("assignelementload", rubeam.cmds.elmload.AssignElementLoads.class);
        kernel.addCmd("assignelementloads", rubeam.cmds.elmload.AssignElementLoads.class);
        kernel.addCmd("removeelementload", rubeam.cmds.elmload.RemoveElementLoad.class);
        kernel.addCmd("setunidistribloadnorvalue", rubeam.cmds.elmload.SetUniDistribLoadNorValue.class);
        kernel.addCmd("setelementloadname", rubeam.cmds.elmload.SetElementLoadName.class);
        kernel.addCmd("setconloadangvalue", rubeam.cmds.elmload.SetConLoadAngValue.class);
        kernel.addCmd("setconloadangangle", rubeam.cmds.elmload.SetConLoadAngAngle.class);
        kernel.addCmd("setconloadangliabs", rubeam.cmds.elmload.SetConLoadAngLiAbs.class);
        kernel.addCmd("setconloadanglirel", rubeam.cmds.elmload.SetConLoadAngLiRel.class);
        // section handling
        kernel.addCmd("removesection", rubeam.cmds.sec.RemoveSection.class);
        // sys commands
        kernel.addCmd("clearstructure", rubeam.cmds.ClearStructure.class);
        kernel.addCmd("rubeaminfo", rubeam.cmds.Info.class);
        kernel.addCmd("importmacro", rubeam.cmds.ImportMacro.class);
        kernel.addCmd("exportmacro", rubeam.cmds.ExportMacro.class);
        kernel.addCmd("addrubeammenu", rubeam.gui.RuBeamMenu.class);

        // Initialize
        RuBeam.initialize();

        // Add menu
        new RuBeamMenu().doCmd(context);

        System.out.println(this);
    }
    public String toString() {
        return "ruBeam plugin successfully loaded (build "+RuBeam.VERSION+")."; 
    }
}
