/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.elm;

import java.util.Iterator;

import rubeam.cmps.ComponentElement;
import rubeam.model.Element;
import cib.cad.db.comp.Component;
import cib.cad.kernel.Kernel;
import cib.util.cmd.Cmd;
import cib.util.cmd.CmdAbortedException;
import cib.util.coll.ClassFilter;
import cib.util.coll.CollUtilities;
import cib.util.coll.ObservableSet;

public class SetElementHingeI implements Cmd {
    
    private boolean m_hinged; // undo data
    private ComponentElement m_cElement = null;
    
    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        ObservableSet<Component> cmpSet = krnl.getDatabase().getComponentSet();
        
        String name = krnl.readString("Enter element name");
        // find node
        Iterator<Component> nit = CollUtilities.filterableIterator(
                cmpSet.iterator(), new ClassFilter(
                        ComponentElement.class));
        Element element = null;
        while (nit.hasNext()) {
            ComponentElement cElement = (ComponentElement) nit.next();
            Element e = cElement.getElement();
            if (name.equals(e.getName())) {
                m_cElement = cElement;
                element = e;
                break;
            }
        }
        if (element != null) {
            String jointed = krnl.readString("Enter jointed i");
            m_hinged = element.getHingeI();
            element.setHingeI(Boolean.parseBoolean(jointed));
            cmpSet.wasChanged(m_cElement);
            System.out.println(this);
        }
        else {
            System.out.println("Element '" + name + "' not found.");
        }
    }

    public void undoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        ObservableSet<Component> cmpSet = krnl.getDatabase().getComponentSet();
        boolean tmp = m_cElement.getElement().getHingeI();
        m_cElement.getElement().setHingeI(m_hinged);
        m_hinged = tmp;
        cmpSet.wasChanged(m_cElement);
    }

    public void redoCmd(Object context) {
        undoCmd(context);
    }
    public boolean isUndoable() {
        return true;
    }
    public boolean changesState() {
        return (m_cElement != null);
    }
    public String toString() {
        return "Set hinge i of element '" + m_cElement.getElement().getName() + "' to '"
            + m_cElement.getElement().getHingeI() + "'";
    }
}
