/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.node;

import java.awt.Window;
import java.awt.geom.Point2D;
import java.util.Set;

import rubeam.RuBeam;
import rubeam.cmps.ComponentNode;
import rubeam.model.Node;
import rubeam.model.Structure;
import cib.cad.db.Database;
import cib.cad.db.att.AttributeManager;
import cib.cad.db.att.Attributes;
import cib.cad.db.comp.Component;
import cib.cad.kernel.Kernel;
import cib.util.CoordSpace;
import cib.util.NameSpace;
import cib.util.cmd.Cmd;
import cib.util.cmd.CmdAbortedException;
import cib.util.geo.Geo2D;
import cib.util.view.ViewUtilities;

/**
 * @author Christian Koch
 */
public class AddNode implements Cmd {

    private ComponentNode m_comp = null;
    private Structure m_structure = null;
    private Node m_node = null;

    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        Database db = krnl.getDatabase();
        CoordSpace cs = CoordSpace.getCoordSpace();
        NameSpace ns = db.getNameSpace();
        Set<Component> cmpSet = db.getComponentSet();
        AttributeManager atm = db.getAttributeManager();
        Attributes attr = atm.getDefaultAttributes();
        m_structure = RuBeam.getStructure();
        ViewUtilities.requestFocusAsynchronously(
                (Window)krnl.getUserInterface());
        

        Point2D ctrU = krnl.readPoint("Enter point");
        Point2D ctrM = cs.getUserCrdTransform().transform(ctrU, null);
        String name;
        if (krnl.hasArgument()) {
            name = krnl.readString("Enter name");
        }  
        else {
            name = ns.generateName(RuBeam.NODE_PREFIX);
        }
        m_node = new Node(ctrM.getX(), ctrM.getY(), name);
        m_structure.addNode(m_node);

        m_comp = new ComponentNode(m_node);

        m_comp.setAttributes(attr);
        cmpSet.add(m_comp);

        System.out.println(this);
    }

    public void undoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        Database db = krnl.getDatabase();
        Set<Component> cmpSet = db.getComponentSet();
        m_structure.removeNode(m_node);
        cmpSet.remove(m_comp);

    }

    public void redoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        Database db = krnl.getDatabase();
        Set<Component> cmpSet = db.getComponentSet();
        m_structure.addNode(m_node);
        cmpSet.add(m_comp);

    }

    public boolean changesState() {
        return true;
    }

    public boolean isUndoable() {
        return true;
    }

    public String toString() {
        return "Added node " + m_node.getName() + " at ("
                + Geo2D.round(m_node.getPoint().getX()) + ", "
                + Geo2D.round(m_node.getPoint().getY()) + ").";
    }
}
