/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.node;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import rubeam.cmps.ComponentNode;
import rubeam.model.Node;
import cib.cad.db.Database;
import cib.cad.db.comp.Component;
import cib.cad.kernel.Kernel;
import cib.util.cmd.Cmd;
import cib.util.cmd.CmdAbortedException;
import cib.util.coll.ClassFilter;
import cib.util.coll.CollUtilities;

public class SetFixed implements Cmd {

    private List<double[]> m_cons = new ArrayList<double[]>();
    private List<ComponentNode> m_nodes = new ArrayList<ComponentNode>();

    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        Database db = krnl.getDatabase();
        Set<Component> compSet = db.getComponentSet();
        Set<Component> selSet = db.getSelectSet();
        
        if (krnl.hasArgument()) {
            String name = krnl.readString("Enter name");
            Iterator<Component> eit = CollUtilities.filterableIterator(
                    compSet.iterator(), new ClassFilter(
                            ComponentNode.class));
            while (eit.hasNext()) {
                ComponentNode cNode = (ComponentNode) eit.next();
                Node node = cNode.getNode();
                if (node.getName().equalsIgnoreCase(name)) {
                    m_nodes.add(cNode);
                    break;
                }
            }
        }
        
        if (m_nodes.isEmpty() && selSet.isEmpty())
            System.err.println("No components selected.");

        if (!m_nodes.isEmpty()) {
            ComponentNode cNode = m_nodes.get(0);
            m_cons.add(cNode.getNode().getContraints());
            cNode.setConstraints(0, 0, 0);
        }
        else {
            Iterator<Component> it = CollUtilities.filterableIterator(selSet
                    .iterator(), new ClassFilter(ComponentNode.class));
            while (it.hasNext()) {
                ComponentNode cNode = (ComponentNode) it.next();
                m_cons.add(cNode.getNode().getContraints());
                cNode.setConstraints(0, 0, 0);
                m_nodes.add(cNode);
            }
        }
        System.out.println(this);
    }

    public void undoCmd(Object context) {
        for (int i=0; i < m_nodes.size(); i++) {
            Node node = m_nodes.get(i).getNode();
            double[] cons = m_cons.get(i);
            node.setContraints(cons[0], cons[1], cons[2]);
        }
    }

    public void redoCmd(Object context) {
        for (ComponentNode cNode : m_nodes) {
            cNode.getNode().setContraints(0, 0, 0);
        }
    }

    public boolean changesState() {
        return true;
    }

    public boolean isUndoable() {
        return true;
    }

    public String toString() {
        String ret = "Fixed contraints set for node(s): ";
        for (int i=0; i < m_nodes.size(); i++) {
            ret += m_nodes.get(i).getNode().getName();
            if (i < (m_nodes.size()-1))
                ret += ", ";
        }
        return ret;

    }
}
