/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.node;

import java.util.Iterator;

import rubeam.cmps.ComponentNode;
import rubeam.model.Node;
import cib.cad.db.comp.Component;
import cib.cad.kernel.Kernel;
import cib.util.cmd.Cmd;
import cib.util.cmd.CmdAbortedException;
import cib.util.coll.ClassFilter;
import cib.util.coll.CollUtilities;
import cib.util.coll.ObservableSet;

public class SetNodeConstraintY implements Cmd {
    
    private double m_dy; // undo data
    private ComponentNode m_cNode = null;
    
    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        ObservableSet<Component> cmpSet = krnl.getDatabase().getComponentSet();
        
        String name = krnl.readString("Enter node name");
        // find node
        Iterator<Component> nit = CollUtilities.filterableIterator(
                cmpSet.iterator(), new ClassFilter(
                        ComponentNode.class));
        Node node = null;
        while (nit.hasNext()) {
            ComponentNode cNode = (ComponentNode) nit.next();
            Node n = cNode.getNode();
            if (name.equals(n.getName())) {
                m_cNode = cNode;
                m_dy = n.getContraints()[1];
                node = n;
                break;
            }
        }
        if (node != null) {
            String textDy = krnl.readString("Enter new dy");
            double newDy = m_dy;
            if (textDy.isEmpty())
                newDy = Double.NaN;
            else {
                try {
                    newDy = Double.parseDouble(textDy);
                }
                catch (NumberFormatException e) {
                    newDy = Double.NaN;
                }
            }
            node.setContraintY(newDy);
            cmpSet.wasChanged(m_cNode);
            System.out.println(this);
        }
    }

    public void undoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        ObservableSet<Component> cmpSet = krnl.getDatabase().getComponentSet();
        double tmp = m_cNode.getNode().getContraints()[1];
        m_cNode.getNode().setContraintY(m_dy);
        m_dy = tmp;
        cmpSet.wasChanged(m_cNode);
    }

    public void redoCmd(Object context) {
        undoCmd(context);
    }
    public boolean isUndoable() {
        return true;
    }
    public boolean changesState() {
        return true;
    }
    public String toString() {
        return "Set dy of node '" + m_cNode.getNode().getName() 
        + "' to " + m_cNode.getNode().getContraints()[1];  
    }
}
