/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.node;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import rubeam.RuBeam;
import rubeam.cmps.ComponentNode;
import rubeam.model.Node;
import rubeam.model.Structure;
import cib.cad.db.Database;
import cib.cad.db.comp.Component;
import cib.cad.kernel.Kernel;
import cib.util.cmd.Cmd;
import cib.util.cmd.CmdAbortedException;
import cib.util.coll.ClassFilter;
import cib.util.coll.CollUtilities;
import cib.util.coll.ObservableSet;

public class SetNodeConstraints implements Cmd {
    
    private Map<ComponentNode, double[]> m_constMap = new HashMap<ComponentNode, double[]>(); // undo data
    private Structure m_structure;
    
    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        Database db = krnl.getDatabase();
        ObservableSet<Component> cmpSet = db.getComponentSet();
        Set<Component> selSet = db.getSelectSet();
        m_structure = RuBeam.getStructure();

        // macro mode
        if (krnl.hasArgument()) {

            String name = krnl.readString("Enter node name");
            Iterator<Component> nit = CollUtilities.filterableIterator(cmpSet
                    .iterator(), new ClassFilter(ComponentNode.class));
            // find node
            while (nit.hasNext()) {
                ComponentNode cNode = (ComponentNode) nit.next();
                Node n = cNode.getNode();
                if (name.equals(n.getName())) {
                    m_constMap.put(cNode, cNode.getNode().getContraints());
                    break;
                }
            }
        }
        // interactive mode
        else {
            // find node
            Iterator<Component> nit = CollUtilities.filterableIterator(selSet
                    .iterator(), new ClassFilter(ComponentNode.class));
            while (nit.hasNext()) {
                ComponentNode cNode = (ComponentNode) nit.next();
                m_constMap.put(cNode, cNode.getNode().getContraints());
            }
        }

        if (!m_constMap.isEmpty()) {
            
            String textDphi = krnl.readString("Enter new constraint phi");
            double newDphi = Double.NaN;
            if (textDphi.isEmpty())
                newDphi = Double.NaN;
            else {
                try {
                    newDphi = Double.parseDouble(textDphi);
                } catch (NumberFormatException e) {
                    newDphi = Double.NaN;
                }
            }

            String textDy = krnl.readString("Enter new constraint y");
            double newDy = Double.NaN;
            if (textDy.isEmpty())
                newDy = Double.NaN;
            else {
                try {
                    newDy = Double.parseDouble(textDy);
                } catch (NumberFormatException e) {
                    newDy = Double.NaN;
                }
            }

            String textDx = krnl.readString("Enter new constraint x");
            double newDx = Double.NaN;
            if (textDx.isEmpty())
                newDx = Double.NaN;
            else {
                try {
                    newDx = -Double.parseDouble(textDx);
                } catch (NumberFormatException e) {
                    newDx = Double.NaN;
                }
            }

            for (ComponentNode cNode : m_constMap.keySet()) {
                Node node = cNode.getNode();
                node.setContraints(newDphi, newDy, newDx);
                cmpSet.wasChanged(cNode);
            }

            System.out.println(this);
            return;
        }
        System.err.println("No node selected!");
    }

    public void undoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        ObservableSet<Component> cmpSet = krnl.getDatabase().getComponentSet();
        
        for (Entry<ComponentNode, double[]> entry : m_constMap.entrySet()) {
            // save old 
            ComponentNode cNode = entry.getKey();
            Node node = cNode.getNode();
            double[] sv = node.getContraints();
            double[] consts = entry.getValue();
            node.setContraints(consts[0], consts[1], consts[2]);
            entry.setValue(sv);
            cmpSet.wasChanged(cNode);
        }
    }

    public void redoCmd(Object context) {
        undoCmd(context);
    }
    public boolean isUndoable() {
        return true;
    }
    public boolean changesState() {
        return true;
    }
    public String toString() {
        return "Set constraints of " + m_constMap.size() + " node(s)."; 
    }
}
