/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.nodeload;

import java.awt.Window;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import rubeam.RuBeam;
import rubeam.cmps.ComponentNodalLoadAng;
import rubeam.cmps.ComponentNode;
import rubeam.model.Node;
import rubeam.model.Structure;
import rubeam.model.loads.NodalLoadAng;
import cib.cad.db.Database;
import cib.cad.db.comp.Component;
import cib.cad.kernel.Kernel;
import cib.util.CoordSpace;
import cib.util.NameSpace;
import cib.util.cmd.Cmd;
import cib.util.cmd.CmdAbortedException;
import cib.util.coll.ClassFilter;
import cib.util.coll.CollUtilities;
import cib.util.geo.Vector2D;
import cib.util.view.ViewUtilities;

public class AddNodalLoadAng implements Cmd {

    List<ComponentNode> m_nodes = new ArrayList<ComponentNode>();
    List<ComponentNodalLoadAng> m_cLoads = new ArrayList<ComponentNodalLoadAng>();
    List<NodalLoadAng> m_loads = new ArrayList<NodalLoadAng>();
    Structure m_structure = null;

    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        Database db = krnl.getDatabase();
        CoordSpace cs = CoordSpace.getCoordSpace();
        Set<Component> compSet = db.getComponentSet();
        Set<Component> selSet = db.getSelectSet();
        NameSpace ns = db.getNameSpace();
        m_structure = RuBeam.getStructure();
        ViewUtilities.requestFocusAsynchronously(
                (Window)krnl.getUserInterface());    

        if (krnl.hasArgument()) {
            String name = krnl.readString("Enter name");
            Iterator<Component> eit = CollUtilities.filterableIterator(compSet
                    .iterator(), new ClassFilter(ComponentNode.class));
            while (eit.hasNext()) {
                ComponentNode cNode = (ComponentNode) eit.next();
                Node node = cNode.getNode();
                if (node.getName().equalsIgnoreCase(name)) {
                    m_nodes.add(cNode);
                    break;
                }
            }
        }

        Point2D pt = null;
        if (m_nodes.isEmpty() && selSet.isEmpty()) {
            pt = krnl.readPoint("Enter point");
        }
        Iterator<Component> it = null;
        if (!selSet.isEmpty()) {
            it = CollUtilities.filterableIterator(selSet.iterator(),
                    new ClassFilter(ComponentNode.class));
            if (pt == null && !it.hasNext())
                pt = krnl.readPoint("Enter point");
        }

        double value = krnl.readDouble("Enter load value", 10000);
        double angle = Math.toRadians(krnl.readDouble("Enter load angle", 0.0));

        Vector2D uAng = new Vector2D(angle);
        Point2D pmAng = cs.getUserCrdTransform().transform(uAng, null);
        angle = Vector2D.X_UNIT.getAngleCCW(new Vector2D(pmAng));
        
        String name = null;
        if (krnl.hasArgument()) {
            name = krnl.readString("Enter name");
        }  
        /*else {
            name = ns.generateName(RuBeam.NODELOAD_PREFIX);
        }*/
        
        if (pt != null) {
            if (name == null)
                name = ns.generateName(RuBeam.NODALLOADANG_PREFIX);
            NodalLoadAng load = new NodalLoadAng(value, angle, name);
            m_structure.addNodeLoad(load); // node.addLoad(load);
            m_loads.add(load);
            ComponentNodalLoadAng cLoad = new ComponentNodalLoadAng(load, pt);
            m_cLoads.add(cLoad);
            compSet.add(cLoad);

        }
        else if (!m_nodes.isEmpty()) {
            Node node = m_nodes.get(0).getNode();
            if (name == null)
                name = ns.generateName(RuBeam.NODALLOADANG_PREFIX);
            NodalLoadAng load = new NodalLoadAng(value, angle, name);
            m_structure.addNodeLoad(load); // node.addLoad(load);
            m_structure.putNodeLoad(node, load);
            m_loads.add(load);

            ComponentNodalLoadAng cLoad = new ComponentNodalLoadAng(load, null);
            m_cLoads.add(cLoad);
            compSet.add(cLoad);
        }
        else {
            while (it.hasNext()) {
                ComponentNode cNode = (ComponentNode) it.next();
                m_nodes.add(cNode);
                name = ns.generateName(RuBeam.NODALLOADANG_PREFIX);
                Node node = cNode.getNode();
                NodalLoadAng load = new NodalLoadAng(value, angle, name);
                m_structure.addNodeLoad(load); // node.addLoad(load);
                m_structure.putNodeLoad(node, load);
                m_loads.add(load);

                ComponentNodalLoadAng cLoad = new ComponentNodalLoadAng(load,
                        null);
                m_cLoads.add(cLoad);
                compSet.add(cLoad);
            }
        }

        System.out.println(this);
    }

    public void undoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        Database db = krnl.getDatabase();
        Set<Component> compSet = db.getComponentSet();

        for (int i = 0; i < m_loads.size(); i++) {
            ComponentNodalLoadAng cForce = m_cLoads.get(i);
            NodalLoadAng load = m_loads.get(i);
            m_structure.removeNodeLoad(load);// cNode.getNode().removeLoad(load);
            compSet.remove(cForce);
        }

        // RuBeam.invalidateStructure();
    }

    public void redoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        Database db = krnl.getDatabase();
        Set<Component> compSet = db.getComponentSet();

        for (int i = 0; i < m_loads.size(); i++) {
            ComponentNode cNode = m_nodes.get(i);
            ComponentNodalLoadAng cForce = m_cLoads.get(i);
            NodalLoadAng load = m_loads.get(i);
            m_structure.addNodeLoad(load);// cNode.getNode().addLoad(load);
            if (cNode != null)
                m_structure.putNodeLoad(cNode.getNode(), load);
            compSet.add(cForce);
        }
        // RuBeam.invalidateStructure();
    }

    public boolean changesState() {
        return true;
    }

    public boolean isUndoable() {
        return true;
    }

    public String toString() {
        String ret = "Load set for node(s): ";
        for (ComponentNode cNode : m_nodes)
            ret += cNode.getNode().getName() + ", ";
        return ret;

    }
}
