/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.nodeload;

import java.util.Iterator;

import rubeam.cmps.ComponentNodalLoadAng;
import rubeam.model.loads.NodalLoadAng;
import cib.cad.db.comp.Component;
import cib.cad.kernel.Kernel;
import cib.util.cmd.Cmd;
import cib.util.cmd.CmdAbortedException;
import cib.util.coll.ClassFilter;
import cib.util.coll.CollUtilities;
import cib.util.coll.ObservableSet;

public class SetNodeLoadAngle implements Cmd {
    
    private double m_a; // undo data [rad]
    private ComponentNodalLoadAng m_cLoad = null;
    
    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        ObservableSet<Component> cmpSet = krnl.getDatabase().getComponentSet();
        
        String name = krnl.readString("Enter node load name");
        // find node
        Iterator<Component> nit = CollUtilities.filterableIterator(
                cmpSet.iterator(), new ClassFilter(
                        ComponentNodalLoadAng.class));
        NodalLoadAng load = null;
        while (nit.hasNext()) {
            ComponentNodalLoadAng cLoad = (ComponentNodalLoadAng) nit.next();
            NodalLoadAng l = (NodalLoadAng)cLoad.getLoad();
            if (name.equals(l.getName())) {
                m_cLoad = cLoad;
                m_a = l.getAngle(); 
                load = l;
                break;
            }
        }
        if (load != null) {
            double newAngle = krnl.readDouble("Enter new angle");
            load.setAngle(Math.toRadians(newAngle));
            cmpSet.wasChanged(m_cLoad);
            System.out.println(this);
        }
        else {
            System.out.println("Node load '" + name + "' not found.");
        }
    }

    public void undoCmd(Object context) {
        Kernel krnl = (Kernel) context;
        ObservableSet<Component> cmpSet = krnl.getDatabase().getComponentSet();
        NodalLoadAng load = (NodalLoadAng)m_cLoad.getLoad();
        double tmp = load.getAngle();
        load.setAngle(m_a);
        m_a = tmp;
        cmpSet.wasChanged(m_cLoad);
    }

    public void redoCmd(Object context) {
        undoCmd(context);
    }
    public boolean isUndoable() {
        return true;
    }
    public boolean changesState() {
        return (m_cLoad != null);
    }
    public String toString() {
        return "Set angle \u03b1 of node load'" + m_cLoad.getLoad().getName() 
            + "' to " + Math.toRadians(
                    ((NodalLoadAng)m_cLoad.getLoad()).getAngle()); 
    }
}
