/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.cmds.sec;

import java.awt.Window;

import rubeam.RuBeam;
import rubeam.model.sections.AbstractSection;
import cib.cad.kernel.Kernel;
import cib.util.cmd.Cmd;
import cib.util.cmd.CmdAbortedException;
import cib.util.view.ViewUtilities;

public abstract class AddSection implements Cmd {

    protected AbstractSection m_section = null;
    
    public void doCmd(Object context) throws CmdAbortedException {
        Kernel krnl = (Kernel) context;
        ViewUtilities.requestFocusAsynchronously(
                (Window)krnl.getUserInterface());
        addSection(context);
    }
    protected abstract void addSection(Object context) throws CmdAbortedException;
    
    public void undoCmd(Object context) {
        RuBeam.getStructure().removeSection(m_section);
    }
    public void redoCmd(Object context) {
        RuBeam.getStructure().addSection(m_section);
    }
    public boolean changesState() {
        return (m_section != null);
    }
    public boolean isUndoable() {
        return true;
    }
    public String toString() {
        return "Added " + m_section.getClass().getSimpleName() + " '"
                + m_section.getName() + "'.";
    }
}
