/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.gui;

import java.awt.Color;
import java.util.List;

import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import rubeam.RuBeam;
import rubeam.model.sections.AbstractSection;
import rubeam.model.sections.LinearGeneralSection;
import cib.cad.kernel.InputDeviceAdapter;
import cib.cad.kernel.TextToken;

public class GeneralSectionTablePanel extends JScrollPane {

    private static final long serialVersionUID = 1L;
    private static final NumberCellRenderer numberRenderer = new NumberCellRenderer();
    private static final String[] colNames = new String[] { "Name", "Ai", "Ak", "Ii", "E"};
    // GUI
    private JTable table = new JTable();
    private TableModel model = new TableModel();

    // Input device
    private InputDeviceAdapter inputDevice = new InputDeviceAdapter();

    public GeneralSectionTablePanel(InputDeviceAdapter inputDevice) {
        super();
        this.inputDevice = inputDevice;
        initialize();
    }

    private void initialize() {
        this.setViewportView(table);
        this.setVerticalScrollBarPolicy(VERTICAL_SCROLLBAR_AS_NEEDED);
        this.setHorizontalScrollBarPolicy(HORIZONTAL_SCROLLBAR_AS_NEEDED);
        table.setModel(model);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.setGridColor(Color.LIGHT_GRAY);
        TableColumn tc = table.getColumnModel().getColumn(1);
        tc.setCellRenderer(numberRenderer);
        tc = table.getColumnModel().getColumn(2);
        tc.setCellRenderer(numberRenderer);
        tc = table.getColumnModel().getColumn(3);
        tc.setCellRenderer(numberRenderer);
        tc = table.getColumnModel().getColumn(4);
        tc.setCellRenderer(numberRenderer);
    }

    public void updatePanel() {
        model.fireTableDataChanged();
    }

    public AbstractSection getSelectedSection() {
        int row = table.getSelectedRow();
        if (row != -1 /*&& row != 0*/)
            return RuBeam.getStructure().readSections(LinearGeneralSection.class).get(row);
        else
            return null;
    }

    class TableModel extends DefaultTableModel {

        private static final long serialVersionUID = 1L;

        public int getColumnCount() {
            return colNames.length;
        }

        public String getColumnName(int column) {
            return colNames[column];
        }

        public int getRowCount() {
            return RuBeam.getStructure().readSections(LinearGeneralSection.class).size();
        }

        public Class<?> getColumnClass(int columnIndex) {
            switch (columnIndex) {
            case 0:
                return String.class;
            case 1:
                return Double.class;
            case 2:
                return Double.class;
            case 3:
                return Double.class;
            case 4:
                return Double.class;
            default:
                throw new IllegalStateException();
            }
        }

        public boolean isCellEditable(int row, int column) {
            if (row == 0)
                return false;
            return true;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            List<AbstractSection> secList = RuBeam.getStructure().readSections(LinearGeneralSection.class);
            if (rowIndex >= secList.size())
                return null;
            LinearGeneralSection sec = 
                (LinearGeneralSection)secList.get(rowIndex);
            switch (columnIndex) {
            case 0:
                return sec.getName();
            case 1:
                return Double.valueOf(sec.getAi());
            case 2:
                return Double.valueOf(sec.getAk());
            case 3:
                return Double.valueOf(sec.getIi());
            case 4:
                return Double.valueOf(sec.getE());
            
            default:
                throw new IllegalStateException();
            }
        }

        public void setValueAt(Object value, int rowIndex, int columnIndex) {
            if (rowIndex == 0)
                return;
            switch (columnIndex) {
            case 0: // set name
                String newName = value.toString();
                List<AbstractSection> secList = RuBeam.getStructure().readSections(LinearGeneralSection.class);
                String oldName = secList.get(rowIndex).getName();
                String fire = "setsectionname " 
                    + TextToken.stringToLiteral(oldName) + " " 
                    + TextToken.stringToLiteral(newName);
                fireInput(fire);
                model.fireTableCellUpdated(rowIndex, columnIndex);
                break;
            case 1: // set Ai
                try {
                    double newAi = Double.parseDouble(value.toString());
                    secList = RuBeam.getStructure().readSections(LinearGeneralSection.class);
                    String secName = secList.get(rowIndex).getName();
                    fire = "setsectionai " 
                        + TextToken.stringToLiteral(secName) + " " + newAi;
                    fireInput(fire);
                } catch (NumberFormatException e) {
                    // do nothing
                }
                model.fireTableCellUpdated(rowIndex, columnIndex);
                break;
            case 2: // set Ak
                try {
                    double newAk = Double.parseDouble(value.toString());
                    secList = RuBeam.getStructure().readSections(LinearGeneralSection.class);
                    String secName = secList.get(rowIndex).getName();
                    fire = "setsectionak " 
                        + TextToken.stringToLiteral(secName) + " " + newAk;
                    fireInput(fire);
                } catch (NumberFormatException e) {
                    // do nothing
                }
                model.fireTableCellUpdated(rowIndex, columnIndex);
                break;
            case 3: // set Ii
                try {
                    double newIi = Double.parseDouble(value.toString());
                    secList = RuBeam.getStructure().readSections(LinearGeneralSection.class);
                    String secName = secList.get(rowIndex).getName();
                    fire = "setsectionii " 
                        + TextToken.stringToLiteral(secName) + " " + newIi;
                    fireInput(fire);
                } catch (NumberFormatException e) {
                    // do nothing
                }
                model.fireTableCellUpdated(rowIndex, columnIndex);
                break;
            case 4: // set E
                try {
                    double newE = Double.parseDouble(value.toString());
                    secList = RuBeam.getStructure().readSections(LinearGeneralSection.class);
                    String secName = secList.get(rowIndex).getName();
                    fire = "setsectione " 
                        + TextToken.stringToLiteral(secName) + " " + newE;
                    fireInput(fire);
                } catch (NumberFormatException e) {
                    // do nothing
                }
                model.fireTableCellUpdated(rowIndex, columnIndex);
                break;
            default:
                throw new IllegalStateException();
            }
        }
    }

    private void fireInput(String input) {
        if (inputDevice.isEnabled())
            inputDevice.fireInterruptingInput(input);

    }
}
