/*
 * Copyright (C) 2009 Emanuel Bombasaro, Christian Koch
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, look at 
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html or write to the Free 
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * Contact address: emanuel.bombasaro@cademia.org, christian.koch@cademia.org
 * Homepage       : http://www.rubeam.cademia.org/
 */

package rubeam.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import rubeam.RuBeam;
import cib.cad.kernel.Kernel;
import cib.cad.view.GraphicalUserInterface;
import cib.cad.view.cmds.AddMenu;
import cib.util.view.ViewUtilities;

public class RuBeamMenu extends AddMenu {

    public static final String NAME = "ruBeam";
    public static JCheckBoxMenuItem s_cbItem = null;
    public static RuBeamInspector s_ruBeamDialog = null;

    protected void _addMenu(Object context) {
        Kernel krnl = (Kernel) context;
        GraphicalUserInterface gui = (GraphicalUserInterface) krnl
                .getUserInterface();
        JMenuBar menuBar = gui.getJMenuBar();

        // check if already exists
        if (containsMenu(menuBar, NAME)) {
            System.err.println("Menu \"" + NAME + "\" already exists");
            return;
        }

        gui.addInputDevice(m_inputDevice);
        m_menu = new JMenu(NAME);

        JMenu menu = new JMenu("Nodes");
        JMenuItem item = new JMenuItem("Add");
        item.addActionListener(new _CmdListener("addnode"));
        menu.add(item);
        item = new JMenuItem("Edit");
        item.addActionListener(new _CmdListener("changefeatures"));
        menu.add(item);
        item = new JMenuItem("Set constraints");
        item.addActionListener(new _CmdListener("setnodeconsts"));
        menu.add(item);

        m_menu.add(menu);

        menu = new JMenu("Elements");
        item = new JMenuItem("Add");
        item.addActionListener(new _CmdListener("addelement"));
        menu.add(item);
        item = new JMenuItem("Edit");
        item.addActionListener(new _CmdListener("changefeatures"));
        menu.add(item);
        /*item = new JMenuItem("Set hinge i");
        item.addActionListener(new _CmdListener("setelementhingei"));
        menu.add(item);
        item = new JMenuItem("Set hinge k");
        item.addActionListener(new _CmdListener("setelementhingek"));
        menu.add(item);*/
        
        m_menu.add(menu);
        
        menu = new JMenu("Sections");
        JMenu menuAdd = new JMenu("Add");
        menu.add(menuAdd);
        item = new JMenuItem("General section");
        item.addActionListener(new _CmdListener("addgensection"));
        menuAdd.add(item);
        item = new JMenuItem("Rectangular section");
        item.addActionListener(new _CmdListener("addrecsection"));
        menuAdd.add(item);
        item = new JMenuItem("I-Profile section");
        item.addActionListener(new _CmdListener("addiprofsection"));
        menuAdd.add(item);
        item = new JMenuItem("Sandwich section");
        item.addActionListener(new _CmdListener("addsandsection"));
        menuAdd.add(item);
        
        item = new JMenuItem("Assign");
        item.addActionListener(new _CmdListener("assignsection"));
        menu.add(item);
        item = new JMenuItem("Remove");
        item.addActionListener(new _CmdListener("removesection"));
        menu.add(item);
        
        m_menu.add(menu);

        m_menu.addSeparator();
        
        menu = new JMenu("Nodal loades");
        menuAdd = new JMenu("Add");
        menu.add(menuAdd);
        item = new JMenuItem("Nodal angular load");
        item.addActionListener(new _CmdListener("addnodalloadang"));
        menuAdd.add(item);
        /*item = new JMenuItem("Nodal moment");
        item.addActionListener(new _CmdListener("addnodalmoment"));
        menu.add(item);*/
        
        item = new JMenuItem("Edit");
        item.addActionListener(new _CmdListener("changefeatures"));
        menu.add(item);
        item = new JMenuItem("Assign");
        item.addActionListener(new _CmdListener("assignnodalloads"));
        menu.add(item);
        
        m_menu.add(menu);

        menu = new JMenu("Element loads");
        menuAdd = new JMenu("Add");
        menu.add(menuAdd);
        item = new JMenuItem("Uni distributed load normal");
        item.addActionListener(new _CmdListener("addelementunidisloadnor"));
        menuAdd.add(item);
        item = new JMenuItem("Concentrated load angular");
        item.addActionListener(new _CmdListener("addelementconload"));
        menuAdd.add(item);
        
        item = new JMenuItem("Edit");
        item.addActionListener(new _CmdListener("changefeatures"));
        menu.add(item);
        item = new JMenuItem("Assign");
        item.addActionListener(new _CmdListener("assignelementloads"));
        menu.add(item);
        
        m_menu.add(menu);
        
        m_menu.addSeparator();
        
        item = new JMenuItem("Analyse");
        item.addActionListener(new _CmdListener("analyse"));
        m_menu.add(item);

        s_cbItem = new JCheckBoxMenuItem("Analyse automatically", RuBeam
                .isAutoAnalysing());
        s_cbItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String cmd = "autoanalyse ";
                if (s_cbItem.isSelected()) {
                    cmd += "on";
                }
                else {
                    cmd += "off";
                }
                m_inputDevice.fireInterruptingInput(cmd);
            }
        });
        m_menu.add(s_cbItem);

        item = new JMenuItem("Visualize step number");
        item.addActionListener(new _CmdListener("setvisualizesteps"));
        m_menu.add(item);
        /*item = new JMenuItem("Results step number");
        item.addActionListener(new _CmdListener("setresultsteps"));
        m_menu.add(item);*/
        
        m_menu.addSeparator();
        
        final JCheckBoxMenuItem toolItem = new JCheckBoxMenuItem("Show inspector", false);
        s_ruBeamDialog = new RuBeamInspector(gui);
        RuBeam.setInspector(s_ruBeamDialog);
        s_ruBeamDialog.setLocation(gui.getUpperLeftLocationOnScreen(null));
        s_ruBeamDialog.addWindowListener(new WindowAdapter() {
            public void windowClosed(WindowEvent e) {
                toolItem.setSelected(false);
            } 
        });
        toolItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                s_ruBeamDialog.setVisible(toolItem.getState());
            }
        });
        toolItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_I, 
                ViewUtilities.ALT + ViewUtilities.SHIFT));
        m_menu.add(toolItem);
        
        m_menu.addSeparator();
        
        item = new JMenuItem("Import macro");
        item.addActionListener(new _CmdListener("importmacro"));
        m_menu.add(item);
        item = new JMenuItem("Export macro");
        item.addActionListener(new _CmdListener("exportmacro"));
        m_menu.add(item);
        
        
        m_menu.addSeparator();
        
        item = new JMenuItem("Info");
        item.addActionListener(new _CmdListener("rubeaminfo"));
        m_menu.add(item);

        
        menuBar.add(m_menu);
    }
    
    /*protected ImageIcon getImage(String path) {
        Kernel krnl = Kernel.getInstance();
        URL pluginURL = krnl.getPluginURL("PLUGIN_RUBEAM");
        if (pluginURL != null) {
            try {
                return new ImageIcon(new URL("jar:" + pluginURL.toString()
                        + "!/" + path));
            } catch (MalformedURLException e) {
                e.printStackTrace();
            }
        }
        URL url = RuBeamMenu.class.getClassLoader().getResource(path);
        if (url != null) {
            return new ImageIcon(url);
        }
        return new ImageIcon();
    }*/

}
