/* 
 * Copyright (C) 2002-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Paint;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

/** 
 * Augments a shape by its attributes.
 * 
 * @author Berthold Firmenich
 * @version $Revision: 1.5 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/03/08 19:01:12 $
 */
public class AttributedShape implements Shape, Cloneable {

    Shape m_shape = null;
    Stroke m_stroke = null;
    Paint m_draw = null;
    Paint m_fill = null;

    // Constructors
    protected AttributedShape() {
    }
    public AttributedShape(Shape shape) {
        m_shape = shape;
        m_stroke = new BasicStroke(0);
        m_draw = Color.black;
        m_fill = null;
    }
    public AttributedShape(AttributedShape as) {
        if (as.m_shape != null)
            m_shape = new AffineTransform().createTransformedShape(as.m_shape);
        m_stroke = as.m_stroke;
        m_draw = as.m_draw;
        m_fill = as.m_fill;
    }
    public AttributedShape(Shape shape, Stroke stroke, Paint draw, Paint fill) {
        m_shape = shape;
        m_stroke = stroke;
        m_draw = draw;
        m_fill = fill;
    }

    // Object implementation
    public Object clone() {
        try {
            AttributedShape clone = (AttributedShape)super.clone();
            if (m_shape != null)
                clone.m_shape = new AffineTransform().createTransformedShape(
                        m_shape);
            clone.m_stroke = m_stroke;
            clone.m_draw = m_draw;
            clone.m_fill = m_fill;
            return clone;
        } catch (CloneNotSupportedException e) {
            throw new InternalError();
        }
    }
    
    // Attributes
    public void setShape(Shape shape) {
        m_shape = shape;
    }
    public Shape getShape() {
        return m_shape;
    }
    public void setDrawPaint(Paint paint) {
        m_draw = paint;
    }
    public Paint getDrawPaint() {
        return m_draw;
    }
    public void setFillPaint(Paint paint) {
        m_fill = paint;
    }
    public Paint getFillPaint() {
        return m_fill;
    }
    public void setStroke(Stroke stroke) {
        m_stroke = stroke;
    }
    public Stroke getStroke() {
        return m_stroke;
    }

    // Shape implementation
    public boolean contains(double x, double y) {
        return m_shape.contains(x, y);
    }
    public boolean contains(double x, double y, double w, double h) {
        return m_shape.contains(x, y, w, h);
    }
    public boolean contains(Point2D p) {
        return m_shape.contains(p);
    }
    public boolean contains(Rectangle2D r) {
        return m_shape.contains(r);
    }
    public Rectangle getBounds() {
        return m_shape.getBounds();
    }
    public Rectangle2D getBounds2D() {
        return m_shape.getBounds2D();
    }
    public PathIterator getPathIterator(AffineTransform at) {
        return m_shape.getPathIterator(at);
    }
    public PathIterator getPathIterator(AffineTransform at, double flatness) {
        return m_shape.getPathIterator(at, flatness);
    }
    public boolean intersects(double x, double y, double w, double h) {
        return m_shape.intersects(x, y, w, h);
    }
    public boolean intersects(Rectangle2D r) {
        return m_shape.intersects(r);
    }
}