/*
 * Copyright (C) 2002-2010 Berthold Firmenich
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Represents an observable object.
 *
 * @author Berthold Firmenich
 * @version $Revision: 1.10 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public class ObservableObjectAdapter implements ObservableObject, Serializable {

    private static final long serialVersionUID = 0L;

    // Attributes
    private Set<Listener> m_listeners = new HashSet<Listener>();

    // Constructors
    public ObservableObjectAdapter() {
    }

    // ObservableObject interface implementation
    public boolean addListener(Listener l) {
        return m_listeners.add(l);
    }
    public boolean removeListener(Listener l) {
        return m_listeners.remove(l);
    }
    public void clearListeners() {
        m_listeners.clear();
    }

    // Object protocol
    public Object clone() throws CloneNotSupportedException {
        Object dst = super.clone();
        ((ObservableObjectAdapter)dst).m_listeners = new HashSet<Listener>();
        _notifyWasCloned((ObservableObject)dst);
        return dst;
    }

    // Notification to be used by derived classes
    protected void _notifyWasChanged() {
        Iterator<Listener> it = m_listeners.iterator();
        while (it.hasNext()) {
            Listener l = it.next();
            l.wasChanged(this);
        }
    }
    protected void _notifyWasCloned(ObservableObject dst) {
        Iterator<Listener> it = m_listeners.iterator();
        while (it.hasNext()) {
            Listener l = it.next();
            l.wasCloned(this, dst);
        }
    }
}