/*
 * Copyright (C) 2002-2010 Berthold Firmenich
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.coll;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Set;

/**
 * The ObservableSet interface. For ObservableObjects the wasChanged() method
 * must not be called externally.
 *
 * @author Berthold Firmenich, Torsten Richter
 * @version $Revision: 1.10 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public interface ObservableSet<E> extends Set<E> {

    public static class Listener<E> implements Serializable {
        private static final long serialVersionUID = 0L;
        public void wasAdded(E o) {
        }
        public void wasChanged(E o) {
        }
        public void wasRemoved(E o) {
        }
        public void wasCloned(E src, E dst) {
        }
    }

    public void wasChanged(E e);

    public void wasCloned(E src, E dst);

    public boolean addListener(Listener<E> l);

    public boolean removeListener(Listener<E> l);

    public void clearListeners();

    public Iterator<Listener<E>> listenerIterator();
}