/* 
 * Copyright (C) 2002-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.coll;

import java.util.Iterator;
import java.util.NoSuchElementException;

/** 
 * A filterable iterator implementation.
 * 
 * @author Bertie Olivier
 * @version $Revision: 1.6 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
class _FilterableIterator<E> implements Iterator<E> {

        private IteratorFilter<E> m_filter;
        private Iterator<E> m_iter;
        private E obj;
        private boolean hasNext;

        protected _FilterableIterator(Iterator<E> iterator, 
                IteratorFilter<E> filter) {
            m_filter = filter;
            m_iter = iterator;
            filterNext();
        }
        public boolean hasNext() {
            return hasNext;
        }
        public E next() {
            if (obj == null)
                throw new NoSuchElementException();
            E tmpObj = obj;
            filterNext();
            return tmpObj;
        }
        public void remove() {
            throw new UnsupportedOperationException();
        }
        // helper method
        private void filterNext() {
            while (m_iter.hasNext()) {
                obj = m_iter.next();
                if (m_filter.matches(obj)) {
                    hasNext = true;
                    return;
                }
            }
            hasNext = false;
        }
}