/*
 * Copyright (C) 2002-2010 Berthold Firmenich
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.coll;

import java.awt.Point;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

/**
 * A filterable listIterator implementation.
 *
 * @author Bertie Olivier
 * @version $Revision: 1.7 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
class _MultiFilterableListIterator<E> extends _FilterableListIterator<E>
implements ListIterator<E>{

    private IteratorFilter<E>[] m_filters;
    private int internalCounter = 0;

    protected _MultiFilterableListIterator(ListIterator<E> iterator,
            IteratorFilter<E>[] filters) {
        super(iterator);
        if(filters.length == 0)
            throw new RuntimeException("need atleast one filter!");
        this.m_filters = filters;
        this.m_filter = filters[0];
        filterNext();
    }

    protected void filterNext() {
        while(internalCounter < m_filters.length){
            super.filterNext();
            if(!hasNext && internalCounter < m_filters.length -1){
                internalCounter++;
                m_filter = m_filters[internalCounter];
                toStart();
                continue;
            }
            break;
        }
    }
    protected void filterPrevious() {
        while(internalCounter >= 0){
            super.filterPrevious();
            if(!hasPrevious && internalCounter > 0){
                internalCounter--;
                m_filter = m_filters[internalCounter];
                toEnd();
                continue;
            }
            break;
        }
    }

    private void toStart(){
        while(m_iter.hasPrevious())
            m_iter.previous();
    }
    private void toEnd(){
        while(m_iter.hasNext())
            m_iter.next();
    }

    public static void main(String[] args) {
        List<Object> testList = new LinkedList<Object>();

        testList.add(new Integer(0));
        testList.add("A");
        testList.add("B");
        testList.add(new Integer(3));
        testList.add(new Double(4.));
        testList.add("E");
        testList.add("F");
        testList.add(new Integer(7));

        System.out.println(testList);

        IteratorFilter<Object>[] filter = new IteratorFilter[] {
                new IteratorFilter<Object>(){
                    public boolean matches(Object o) {
                        return o instanceof Integer;
                    }
                    public String toString() {
                        return "Integer";
                    }
                },
                new IteratorFilter<Object>(){
                    public boolean matches(Object o) {
                        return o instanceof String;
                    }
                    public String toString() {
                        return "String";
                    }
                },
                new IteratorFilter<Object>(){
                    public boolean matches(Object o) {
                        return o instanceof Integer;
                    }
                    public String toString() {
                        return "Integer";
                    }
                },
                new IteratorFilter<Object>(){
                    public boolean matches(Object o) {
                        return o instanceof Point;
                    }
                    public String toString() {
                        return "Point";
                    }
                },
                new IteratorFilter<Object>(){
                    public boolean matches(Object o) {
                        return o instanceof Double;
                    }
                    public String toString() {
                        return "Double";
                    }
                }
        };

        ListIterator<Object> i = new _MultiFilterableListIterator<Object>(
                testList.listIterator(), filter);

        System.out.print("["+i.previousIndex()+"|"+i.nextIndex()+"]->");
        while(i.hasNext())
            System.out.print(" "+i.next()+"<-["+i.previousIndex()+"|"+
                    i.nextIndex()+"]->");
        System.out.print("["+i.previousIndex()+"]"+"["+i.nextIndex()+"]->");
        System.out.println("\n");


        System.out.print("["+i.previousIndex()+"|"+i.nextIndex()+"]->");
        while(i.hasPrevious())
            System.out.print(" "+i.previous()+"<-["+i.previousIndex()+"|"+
                    i.nextIndex()+"]->");
        System.out.print("["+i.previousIndex()+"|"+i.nextIndex()+"]->");
        System.out.println("\n");

    }
}