/* 
 * Copyright (C) 2002-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.dxf;

/** 
 * An enumeration that represents all valid DXF Group Codes.
 * 
 * @author Berthold Firmenich
 * @version $Revision: 1.4 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public enum GroupCode {
    TYPE                               (0),     GC0(0),
    TEXT                               (1),     GC1(1),
    NAME                               (2),     GC2(2),
    TEXT_OR_NAME_2                     (3),     GC3(3),
    TEXT_OR_NAME_3                     (4),     GC4(4),
    HANDLE                             (5),     GC5(5),
    LINETYPE_NAME                      (6),     GC6(6),
    TEXT_STYLE_NAME                    (7),     GC7(7),
    LAYER_NAME                         (8),     GC8(8),
    VARIABLE_NAME                      (9),     GC9(9),
    
    X_1                               (10),     GC10(10),
    X_2                               (11),     GC11(11),
    X_3                               (12),     GC12(12),
    X_4                               (13),     GC13(13),
    X_5                               (14),     GC14(14),
    X_6                               (15),     GC15(15),
    X_7                               (16),     GC16(16),
    X_8                               (17),     GC17(17),
    X_9                               (18),     GC18(18),
    
    Y_1                               (20),     GC20(20),
    Y_2                               (21),     GC21(21),
    Y_3                               (22),     GC22(22),
    Y_4                               (23),     GC23(23),
    Y_5                               (24),     GC24(24),
    Y_6                               (25),     GC25(25),
    Y_7                               (26),     GC26(26),
    Y_8                               (27),     GC27(27),
    Y_9                               (28),     GC28(28),
    
    Z_1                               (30),     GC30(30),
    Z_2                               (31),     GC31(31),
    Z_3                               (32),     GC32(32),
    Z_4                               (33),     GC33(33),
    Z_5                               (34),     GC34(34),
    Z_6                               (35),     GC35(35),
    Z_7                               (36),     GC36(36),
    Z_8                               (37),     GC37(37),
    
    ELEVATION                         (38),     GC38(38),
    THICKNESS                         (39),     GC39(39),
    
    DOUBLE_1                          (40),     GC40(40),
    DOUBLE_2                          (41),     GC41(41),
    DOUBLE_3                          (42),     GC42(42),
    DOUBLE_4                          (43),     GC43(43),
    DOUBLE_5                          (44),     GC44(44),
    DOUBLE_6                          (45),     GC45(45),
    DOUBLE_7                          (46),     GC46(46),
    DOUBLE_8                          (47),     GC47(47),
    
    LINETYPE_SCALE                    (48),     GC48(48),
    REPEATED_DOUBLE_VALUE             (49),     GC49(49),
    
    ANGLE_1                           (50),     GC50(50),
    ANGLE_2                           (51),     GC51(51),
    ANGLE_3                           (52),     GC52(52),
    ANGLE_4                           (53),     GC53(53),
    ANGLE_5                           (54),     GC54(54),
    ANGLE_6                           (55),     GC55(55),
    ANGLE_7                           (56),     GC56(56),
    ANGLE_8                           (57),     GC57(57),
    ANGLE_9                           (58),     GC58(58),
    
    VISIBILITY                        (60),     GC60(60),
                                                GC61(61),
    COLOR                             (62),     GC62(62),
                                                GC63(63),
                                                GC64(64),
                                                GC65(65),
    ENTITIES_FOLLOW_FLAG              (66),     GC66(66),
    MODEL_OR_PAPER_SPACE              (67),     GC67(67),
    VIEWPORT_DATA_1                   (68),     GC68(68),
    VIEWPORT_DATA_2                   (69),     GC69(69),
    
    INT_1                             (70),     GC70(70),
    INT_2                             (71),     GC71(71),
    INT_3                             (72),     GC72(72),
    INT_4                             (73),     GC73(73),
    INT_5                             (74),     GC74(74),
    INT_6                             (75),     GC75(75),
    INT_7                             (76),     GC76(76),
    INT_8                             (77),     GC77(77),
    INT_9                             (78),     GC78(78),
    INT_10                            (79),     GC79(79),
    
    INT32_1                           (90),     GC90(90),
    INT32_2                           (91),     GC91(91),
    INT32_3                           (92),     GC92(92),
    INT32_4                           (93),     GC93(93),
    INT32_5                           (94),     GC94(94),
    INT32_6                           (95),     GC95(95),
    INT32_7                           (96),     GC96(96),
    INT32_8                           (97),     GC97(97),
    INT32_9                           (98),     GC98(98),
    INT32_10                          (99),     GC99(99),
    
    SUBCLASS_DATA_MARKER             (100),     GC100(100),
    GROUP_CONTROL_STRING             (102),     GC102(102),
    DIMVAR_OBJECT_HANDLE             (105),     GC105(105),
    
    UCS_ORIGIN_X                     (110),     GC110(110), 
    UCS_X_AXIS_X                     (111),     GC111(111),
    UCS_Y_AXIS_X                     (112),     GC112(112),
    UCS_ORIGIN_Y                     (120),     GC120(120),
    UCS_X_AXIS_Y                     (121),     GC121(121),
    UCS_Y_AXIS_Y                     (122),     GC122(122),
    UCS_ORIGIN_Z                     (130),     GC130(130),
    UCS_X_AXIS_Z                     (131),     GC131(131),
    UCS_Y_AXIS_Z                     (132),     GC132(132),
    
    DOUBLE_10                        (140),     GC140(140),
    DOUBLE_11                        (141),     GC141(141),
    DOUBLE_12                        (142),     GC142(142),
    DOUBLE_13                        (143),     GC143(143),
    DOUBLE_14                        (144),     GC144(144),
    DOUBLE_15                        (145),     GC145(145),
    DOUBLE_16                        (146),     GC146(146),
    DOUBLE_17                        (147),     GC147(147),
    DOUBLE_18                        (148),     GC148(148),
    DOUBLE_19                        (149),     GC149(149),
    
    INT16_1                          (170),     GC170(170),
    INT16_2                          (171),     GC171(171),
    INT16_3                          (172),     GC172(172),
    INT16_4                          (173),     GC173(173),
    INT16_5                          (174),     GC174(174),
    INT16_6                          (175),     GC175(175),
    INT16_7                          (176),     GC176(176),
    INT16_8                          (177),     GC177(177),
    INT16_9                          (178),     GC178(178),
    INT16_10                         (179),     GC179(179),
    
    EXTRUSION_X                      (210),     GC210(210),
                                                GC211(211),
                                                GC212(212),
                                                GC213(213),

    EXTRUSION_Y                      (220),     GC220(220),
                                                GC221(221),
                                                GC222(222),
                                                GC223(223),
    
    EXTRUSION_Z                      (230),     GC230(230),
                                                GC231(231),
                                                GC232(232),
                                                GC233(233),
    
    INT16_11                         (270),     GC270(270),
    INT16_12                         (271),     GC271(271),
    INT16_13                         (272),     GC272(272),
    INT16_14                         (273),     GC273(273),
    INT16_15                         (274),     GC274(274),
    INT16_16                         (275),     GC275(275),
    INT16_17                         (276),     GC276(276),
    INT16_18                         (277),     GC277(277),
    INT16_19                         (278),     GC278(278),
    INT16_20                         (279),     GC279(279),
    INT16_21                         (280),     GC280(280),
    INT16_22                         (281),     GC281(281),
    INT16_23                         (282),     GC282(282),
    INT16_24                         (283),     GC283(283),
    INT16_25                         (284),     GC284(284),
    INT16_26                         (285),     GC285(285),
    INT16_27                         (286),     GC286(286),
    INT16_28                         (287),     GC287(287),
    INT16_29                         (288),     GC288(288),
    INT16_30                         (289),     GC289(289),
    
    BOOLEAN_1                        (290),     GC290(290),
    BOOLEAN_2                        (291),     GC291(291),
    BOOLEAN_3                        (292),     GC292(292),
    BOOLEAN_4                        (293),     GC293(293),
    BOOLEAN_5                        (294),     GC294(294),
    BOOLEAN_6                        (295),     GC295(295),
    BOOLEAN_7                        (296),     GC296(296),
    BOOLEAN_8                        (297),     GC297(297),
    BOOLEAN_9                        (298),     GC298(298),
    BOOLEAN_10                       (299),     GC299(299),
    
    ARBITRARY_TEXT_1                 (300),     GC300(300),
    ARBITRARY_TEXT_2                 (301),     GC301(301),
    ARBITRARY_TEXT_3                 (302),     GC302(302),
    ARBITRARY_TEXT_4                 (303),     GC303(303),
    ARBITRARY_TEXT_5                 (304),     GC304(304),
    ARBITRARY_TEXT_6                 (305),     GC305(305),
    ARBITRARY_TEXT_7                 (306),     GC306(306),
    ARBITRARY_TEXT_8                 (307),     GC307(307),
    ARBITRARY_TEXT_9                 (308),     GC308(308),
    ARBITRARY_TEXT_10                (309),     GC309(309),

    ARBITRARY_BINARY_CHUNK_1         (310),     GC310(310),
    ARBITRARY_BINARY_CHUNK_2         (311),     GC311(311),
    ARBITRARY_BINARY_CHUNK_3         (312),     GC312(312),
    ARBITRARY_BINARY_CHUNK_4         (313),     GC313(313),
    ARBITRARY_BINARY_CHUNK_5         (314),     GC314(314),
    ARBITRARY_BINARY_CHUNK_6         (315),     GC315(315),
    ARBITRARY_BINARY_CHUNK_7         (316),     GC316(316),
    ARBITRARY_BINARY_CHUNK_8         (317),     GC317(317),
    ARBITRARY_BINARY_CHUNK_9         (318),     GC318(318),
    ARBITRARY_BINARY_CHUNK_10        (319),     GC319(319),
    
    ARBITRARY_OBJECT_HANDLE_1        (320),     GC320(320),
    ARBITRARY_OBJECT_HANDLE_2        (321),     GC321(321),
    ARBITRARY_OBJECT_HANDLE_3        (322),     GC322(322),
    ARBITRARY_OBJECT_HANDLE_4        (323),     GC323(323),
    ARBITRARY_OBJECT_HANDLE_5        (324),     GC324(324),
    ARBITRARY_OBJECT_HANDLE_6        (325),     GC325(325),
    ARBITRARY_OBJECT_HANDLE_7        (326),     GC326(326),
    ARBITRARY_OBJECT_HANDLE_8        (327),     GC327(327),
    ARBITRARY_OBJECT_HANDLE_9        (328),     GC328(328),
    ARBITRARY_OBJECT_HANDLE_10       (329),     GC329(329),
    
    SOFT_POINTER_HANDLE_1            (330),     GC330(330),
    SOFT_POINTER_HANDLE_2            (331),     GC331(331),
    SOFT_POINTER_HANDLE_3            (332),     GC332(332),
    SOFT_POINTER_HANDLE_4            (333),     GC333(333),
    SOFT_POINTER_HANDLE_5            (334),     GC334(334),
    SOFT_POINTER_HANDLE_6            (335),     GC335(335),
    SOFT_POINTER_HANDLE_7            (336),     GC336(336),
    SOFT_POINTER_HANDLE_8            (337),     GC337(337),
    SOFT_POINTER_HANDLE_9            (338),     GC338(338),
    SOFT_POINTER_HANDLE_10           (339),     GC339(339),
    
    HARD_POINTER_HANDLE_1            (340),     GC340(340),
    HARD_POINTER_HANDLE_2            (341),     GC341(341),
    HARD_POINTER_HANDLE_3            (342),     GC342(342),
    HARD_POINTER_HANDLE_4            (343),     GC343(343),
    HARD_POINTER_HANDLE_5            (344),     GC344(344),
    HARD_POINTER_HANDLE_6            (345),     GC345(345),
    HARD_POINTER_HANDLE_7            (346),     GC346(346),
    HARD_POINTER_HANDLE_8            (347),     GC347(347),
    HARD_POINTER_HANDLE_9            (348),     GC348(348),
    HARD_POINTER_HANDLE_10           (349),     GC349(349),
    
    SOFT_OWNER_HANDLE_1              (350),     GC350(350),
    SOFT_OWNER_HANDLE_2              (351),     GC351(351),
    SOFT_OWNER_HANDLE_3              (352),     GC352(352),
    SOFT_OWNER_HANDLE_4              (353),     GC353(353),
    SOFT_OWNER_HANDLE_5              (354),     GC354(354),
    SOFT_OWNER_HANDLE_6              (355),     GC355(355),
    SOFT_OWNER_HANDLE_7              (356),     GC356(356),
    SOFT_OWNER_HANDLE_8              (357),     GC357(357),
    SOFT_OWNER_HANDLE_9              (358),     GC358(358),
    SOFT_OWNER_HANDLE_10             (359),     GC359(359),
    
    HARD_OWNER_HANDLE_1              (360),     GC360(360),
    HARD_OWNER_HANDLE_2              (361),     GC361(361),
    HARD_OWNER_HANDLE_3              (362),     GC362(362),
    HARD_OWNER_HANDLE_4              (363),     GC363(363),
    HARD_OWNER_HANDLE_5              (364),     GC364(364),
    HARD_OWNER_HANDLE_6              (365),     GC365(365),
    HARD_OWNER_HANDLE_7              (366),     GC366(366),
    HARD_OWNER_HANDLE_8              (367),     GC367(367),
    HARD_OWNER_HANDLE_9              (368),     GC368(368),
    HARD_OWNER_HANDLE_10             (369),     GC369(369),
    
    LINEWEIGHT_1                     (370),     GC370(370),
    LINEWEIGHT_2                     (371),     GC371(371),
    LINEWEIGHT_3                     (372),     GC372(372),
    LINEWEIGHT_4                     (373),     GC373(373),
    LINEWEIGHT_5                     (374),     GC374(374),
    LINEWEIGHT_6                     (375),     GC375(375),
    LINEWEIGHT_7                     (376),     GC376(376),
    LINEWEIGHT_8                     (377),     GC377(377),
    LINEWEIGHT_9                     (378),     GC378(378),
    LINEWEIGHT_10                    (379),     GC379(379),
    
    PLOTSTYLE_NAME_TYPE_1            (380),     GC380(380),
    PLOTSTYLE_NAME_TYPE_2            (381),     GC381(381),
    PLOTSTYLE_NAME_TYPE_3            (382),     GC382(382),
    PLOTSTYLE_NAME_TYPE_4            (383),     GC383(383),
    PLOTSTYLE_NAME_TYPE_5            (384),     GC384(384),
    PLOTSTYLE_NAME_TYPE_6            (385),     GC385(385),
    PLOTSTYLE_NAME_TYPE_7            (386),     GC386(386),
    PLOTSTYLE_NAME_TYPE_8            (387),     GC387(387),
    PLOTSTYLE_NAME_TYPE_9            (388),     GC388(388),
    PLOTSTYLE_NAME_TYPE_10           (389),     GC389(389),
    
    PLOTSTYLE_NAME_HANDLE_1          (390),     GC390(390),
    PLOTSTYLE_NAME_HANDLE_2          (391),     GC391(391),
    PLOTSTYLE_NAME_HANDLE_3          (392),     GC392(392),
    PLOTSTYLE_NAME_HANDLE_4          (393),     GC393(393),
    PLOTSTYLE_NAME_HANDLE_5          (394),     GC394(394),
    PLOTSTYLE_NAME_HANDLE_6          (395),     GC395(395),
    PLOTSTYLE_NAME_HANDLE_7          (396),     GC396(396),
    PLOTSTYLE_NAME_HANDLE_8          (397),     GC397(397),
    PLOTSTYLE_NAME_HANDLE_9          (398),     GC398(398),
    PLOTSTYLE_NAME_HANDLE_10         (399),     GC399(399),
    
    INT16_31                         (400),     GC400(400),
    INT16_32                         (401),     GC401(401),
    INT16_33                         (402),     GC402(402),
    INT16_34                         (403),     GC403(403),
    INT16_35                         (404),     GC404(404),
    INT16_36                         (405),     GC405(405),
    INT16_37                         (406),     GC406(406),
    INT16_38                         (407),     GC407(407),
    INT16_39                         (408),     GC408(408),
    INT16_40                         (409),     GC409(409),
    
    STRING_1                         (410),     GC410(410),
    STRING_2                         (411),     GC411(411),
    STRING_3                         (412),     GC412(412),
    STRING_4                         (413),     GC413(413),
    STRING_5                         (414),     GC414(414),
    STRING_6                         (415),     GC415(415),
    STRING_7                         (416),     GC416(416),
    STRING_8                         (417),     GC417(417),
    STRING_9                         (418),     GC418(418),
    STRING_10                        (419),     GC419(419),
    
    INT32_11                         (420),     GC420(420),
    INT32_12                         (421),     GC421(421),
    INT32_13                         (422),     GC422(422),
    INT32_14                         (423),     GC423(423),
    INT32_15                         (424),     GC424(424),
    INT32_16                         (425),     GC425(425),
    INT32_17                         (426),     GC426(426),
    INT32_18                         (427),     GC427(427),
    
    STRING_11                        (430),     GC430(430),
    STRING_12                        (431),     GC431(431),
    STRING_13                        (432),     GC432(432),
    STRING_14                        (433),     GC433(433),
    STRING_15                        (434),     GC434(434),
    STRING_16                        (435),     GC435(435),
    STRING_17                        (436),     GC436(436),
    STRING_18                        (437),     GC437(437),
    
    INT32_19                         (440),     GC440(440),
    INT32_20                         (441),     GC441(441),
    INT32_21                         (442),     GC442(442),
    INT32_22                         (443),     GC443(443),
    INT32_23                         (444),     GC444(444),
    INT32_24                         (445),     GC445(445),
    INT32_25                         (446),     GC446(446),
    INT32_26                         (447),     GC447(447),
    
    LONG_1                           (450),     GC450(450),
    LONG_2                           (451),     GC451(451),
    LONG_3                           (452),     GC452(452),
    LONG_4                           (453),     GC453(453),
    LONG_5                           (454),     GC454(454),
    LONG_6                           (455),     GC455(455),
    LONG_7                           (456),     GC456(456),
    LONG_8                           (457),     GC457(457),
    LONG_9                           (458),     GC458(458),
    LONG_10                          (459),     GC459(459),
    
    DOUBLE_20                        (460),     GC460(460),
    DOUBLE_21                        (461),     GC461(461),
    DOUBLE_22                        (462),     GC462(462),
    DOUBLE_23                        (463),     GC463(463),
    DOUBLE_24                        (464),     GC464(464),
    DOUBLE_25                        (465),     GC465(465),
    DOUBLE_26                        (466),     GC466(466),
    DOUBLE_27                        (467),     GC467(467),
    DOUBLE_28                        (468),     GC468(468),
    DOUBLE_29                        (469),     GC469(469),
    
    STRING_19                        (470),     GC470(470),
    STRING_20                        (471),     GC471(471),
    STRING_21                        (472),     GC472(472),
    STRING_22                        (473),     GC473(473),
    STRING_23                        (474),     GC474(474),
    STRING_24                        (475),     GC475(475),
    STRING_25                        (476),     GC476(476),
    STRING_26                        (477),     GC477(477),
    STRING_27                        (478),     GC478(478),
    STRING_28                        (479),     GC479(479),

    COMMENT                          (999),     GC999(999),
    
    XDATA_ASCII_STRING              (1000),     GC1000(1000),
    XDATA_APPLICATION_NAME          (1001),     GC1001(1001),
    XDATA_CONTROL_STRING            (1002),     GC1002(1002),
    XDATA_LAYER_NAME                (1003),     GC1003(1003),
    XDATA_CHUNK_OF_BYTES            (1004),     GC1004(1004),
    XDATA_ENTITY_HANDLE             (1005),     GC1005(1005),
    XDATA_X_1                       (1010),     GC1010(1010),
    XDATA_X_2                       (1011),     GC1011(1011),
    XDATA_X_3                       (1012),     GC1012(1012),
    XDATA_X_4                       (1013),     GC1013(1013),
    XDATA_Y_1                       (1020),     GC1020(1020),
    XDATA_Y_2                       (1021),     GC1021(1021),
    XDATA_Y_3                       (1022),     GC1022(1022),
    XDATA_Y_4                       (1023),     GC1023(1023),
    XDATA_Z_1                       (1030),     GC1030(1030),
    XDATA_Z_2                       (1031),     GC1031(1031),
    XDATA_Z_3                       (1032),     GC1032(1032),
    XDATA_Z_4                       (1033),     GC1033(1033),
    XDATA_DOUBLE                    (1040),     GC1040(1040),
    XDATA_DISTANCE                  (1041),     GC1041(1041),
    XDATA_SCALE_FACTOR              (1042),     GC1042(1042),
    XDATA_INT16                     (1070),     GC1070(1070),
    XDATA_INT32                     (1071),     GC1071(1071);
   
    private int m_code;
    
    GroupCode(int code) {
        m_code = code;
    }

    /**
     * The Group Code as Integer. 
     *
     * @return the Group Code as Integer value
     */
    public int toInt() {
        return m_code;
    }
}
