/*
 * Copyright (C) 2002-2010 Berthold Firmenich
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.featureLogic;

import java.sql.ResultSet;
import java.sql.Statement;
import java.util.NoSuchElementException;

/**
 * Implementation of the ResultSetIterator interface.
 *
 * @author Berthold Firmenich
 * @version $Revision: 1.7 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public class ResultSetIteratorImp implements ResultSetIterator{

    private Statement m_statement = null;
    private ResultSet m_resultSet = null;
    private boolean m_cursorMoved = false;

    public ResultSetIteratorImp(Statement statement, ResultSet resultSet) {
        m_statement = statement;
        m_resultSet = resultSet;
    }

    @Override
    protected void finalize() {
        close();
    }

    public void close() {
        try {
            if (m_resultSet != null) {
                m_resultSet.close();
                m_resultSet = null;
            }
            if (m_statement != null) {
                m_statement.close();
                m_statement = null;
            }
        } catch (Exception e) {
        }
    }

    public boolean hasNext() {
        if (m_statement == null || m_resultSet == null)
            return false;

        if (m_cursorMoved)
            return true;

        try {
            if (m_resultSet.next()) {
                m_cursorMoved = true;
                return true;
            }

            m_resultSet.close();
            m_statement.close();
            m_statement = null;
            m_resultSet = null;
        } catch (Exception e) {
        }

        return false;
    }

    public String next() throws NoSuchElementException {
        return nextElement();
    }

    public String nextElement() throws NoSuchElementException {
        if (m_resultSet == null)
            throw new NoSuchElementException();

        try {
            hasNext();
            m_cursorMoved = false;
            return m_resultSet.getString(1);
        } catch (Exception e) {
            throw new NoSuchElementException(e.getMessage());
        }
    }

    public void remove() throws UnsupportedOperationException {
        throw new UnsupportedOperationException();
    }
}