/*
 * Copyright (C) 2002-2010 Berthold Firmenich
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.featureLogic.db.postgresql;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import cib.util.coll.CollUtilities;
import cib.util.coll.ReverseAccessMap;
import cib.util.featureLogic.FeatureLogicAdapter;
import cib.util.featureLogic.FeatureLogicException;
import cib.util.featureLogic.FormattedDate;
import cib.util.featureLogic.ResultSetIterator;
import cib.util.featureLogic.ResultSetIteratorImp;
import cib.util.featureLogic.SetDsc;

/**
 * Implementation of the FeatureLogic class for PostgreSQL databases.
 *
 * @author Berthold Firmenich, Torsten Richter
 * @version $Revision: 1.8 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public class FeatureLogicPostgreSQL extends FeatureLogicAdapter {

    protected Connection m_con = null;
    PreparedStatement pstmtAddElement;
//  PreparedStatement pstmtIsElement;
//  PreparedStatement pstmtIsFeature;
    PreparedStatement pstmtNextID;
    PreparedStatement pstmtSetNextID;
    PreparedStatement pstmtDefineFeature;
    PreparedStatement pstmtAddFeature;
    PreparedStatement pstmtChangeFeature;
    PreparedStatement pstmtFindAtomElement;
    PreparedStatement pstmtInsertIntoAtom;
    PreparedStatement pstmtIsUsedAsPrimitiveElement;
    PreparedStatement pstmtGetFeatureValue;
    PreparedStatement pstmtGetFeatureValues;
    PreparedStatement pstmtGetAtom;

    int countPstmtAddElement;
    int countPstmtAddFeature;
    int countPstmtChangeFeature;
    int countPstmtInsertIntoAtom;

    long m_nextId = 1;
    Set<String> m_domain = new HashSet<String>();
    Set<String> m_feature = new HashSet<String>();
//    Map<String,String> m_reverseAtom = new HashMap<String,String>();
    ReverseAccessMap<String,String> m_atom =
        CollUtilities.reverseAccessMap(new HashMap<String,String>());

    boolean m_batchMode = false;


   /**
    * @param con the JDBC database connection
    */
    public FeatureLogicPostgreSQL(Connection con) {
        m_con = con;

        try {
            pstmtAddElement = m_con.prepareStatement(
                "INSERT INTO Domain VALUES (?)");

//            pstmtIsElement = m_con.prepareStatement(
//                "SELECT Count(*) AS n FROM Domain " +
//                "WHERE Domain.element=?");

//            pstmtIsFeature = m_con.prepareStatement(
//                "SELECT Count(*) AS n FROM Feature " +
//                "WHERE Feature.feature=?");

            pstmtNextID = m_con.prepareStatement(
                "SELECT id FROM NextAvailableId");

            pstmtSetNextID = m_con.prepareStatement(
                "UPDATE NextAvailableId SET id=? WHERE id=?");

            pstmtDefineFeature = m_con.prepareStatement(
                "INSERT INTO Feature VALUES (?)");

            pstmtAddFeature = m_con.prepareStatement(
                "INSERT INTO RelSlot (object, feature, value) " +
                "VALUES (?,?,?)");

            pstmtChangeFeature = m_con.prepareStatement(
                "UPDATE RelSlot SET value=? " +
                "WHERE object=? AND feature=?");

            pstmtFindAtomElement = m_con.prepareStatement(
                "SELECT element FROM Atom WHERE value = ?");

            pstmtInsertIntoAtom = m_con.prepareStatement(
                "INSERT INTO Atom (element, value) " +
                "VALUES (?,?)");

            pstmtIsUsedAsPrimitiveElement = m_con.prepareStatement(
                "SELECT Count(*) AS n FROM Atom " +
                "WHERE Atom.element=?");

            pstmtGetFeatureValue = m_con.prepareStatement(
                "SELECT feature, value FROM RelSlot " +
                "WHERE object=? AND feature=?");

            pstmtGetFeatureValues = m_con.prepareStatement(
                "SELECT feature, value FROM RelSlot WHERE object=?");

            pstmtGetAtom = m_con.prepareStatement(
                "SELECT value FROM Atom WHERE element=?");

            m_nextId = _getNextIdFromDB();

            String sql = "SELECT * FROM Domain";
            Statement stmt = m_con.createStatement();
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next()) {
                m_domain.add(rs.getString(1));
            }
            stmt.close();

            sql = "SELECT * FROM Feature";
            stmt = m_con.createStatement();
            rs = stmt.executeQuery(sql);
            while (rs.next()) {
                m_feature.add(rs.getString(1));
            }
            stmt.close();

            sql = "SELECT * FROM Atom";
            stmt = m_con.createStatement();
            rs = stmt.executeQuery(sql);
            while (rs.next()) {
                m_atom.put(rs.getString(1), rs.getString(2));
            }
            stmt.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

//  ------------------------------------------------------------------------
//  Data manipulation
//  ------------------------------------------------------------------------

    public boolean isBatchMode() {
        return m_batchMode;
    }

    public void setBatchMode(boolean batchMode) {
        m_batchMode = batchMode;
        /*try {
            if (m_batchMode)
                m_con.setAutoCommit(false);
            else
                m_con.setAutoCommit(true);
        } catch (SQLException e) {
            System.err.println(e.getMessage());
        }*/
    }

    public void executeBatch() throws SQLException {
        if (m_batchMode) {
            try {
                _setNextId();
                System.out.println("Batch: " +
                    countPstmtAddElement + " new elements, " +
                    countPstmtInsertIntoAtom + " new atoms, " +
                    countPstmtAddFeature + " added and " +
                    countPstmtChangeFeature + " changed feature values.");

                m_con.setAutoCommit(false);
                //System.err.println("Add all elements.");
                if (countPstmtAddElement > 0) {
                    pstmtAddElement.executeBatch();
                    m_con.commit();
                }
                //System.err.println("Add all atoms.");
                if (countPstmtInsertIntoAtom > 0) {
                    pstmtInsertIntoAtom.executeBatch();
                    m_con.commit();
                }
                //System.err.println("Add all features.");
                if (countPstmtAddFeature > 0) {
                    pstmtAddFeature.executeBatch();
                    m_con.commit();
                }
                //System.err.println("Change features.");
                if (countPstmtChangeFeature > 0) {
                    pstmtChangeFeature.executeBatch();
                    m_con.commit();
                }
                m_con.setAutoCommit(true);

                countPstmtAddElement = 0;
                countPstmtAddFeature = 0;
                countPstmtChangeFeature = 0;
                countPstmtInsertIntoAtom = 0;
                pstmtAddElement.clearBatch();
                pstmtAddFeature.clearBatch();
                pstmtChangeFeature.clearBatch();
                pstmtInsertIntoAtom.clearBatch();
            } catch (SQLException e) {
                m_con.setAutoCommit(true);
                throw e;
            }
        }
    }

    public void clear(boolean onlyTransient) throws SQLException {
        if (m_con == null)
            return;

        if (!onlyTransient) {
            Statement stmt = null;
            stmt = m_con.createStatement();
            stmt.addBatch("DELETE FROM ATOM");
            stmt.addBatch("DELETE FROM DOMAIN");
            stmt.addBatch("DELETE FROM FEATURE");
            stmt.addBatch("DELETE FROM RELSLOT");
            stmt.addBatch("DELETE FROM NextAvailableId");
            stmt.addBatch("INSERT INTO NextAvailableId (ID) VALUES (1)");
            stmt.executeBatch();
            stmt.close();
        }

        m_domain.clear();
        m_feature.clear();
        m_atom.clear();
        m_nextId = 1;
        countPstmtAddElement = 0;
        countPstmtAddFeature = 0;
        countPstmtChangeFeature = 0;
        countPstmtInsertIntoAtom = 0;
    }

    private long _getNextIdFromDB() throws SQLException {
        ResultSet rs = pstmtNextID.executeQuery();
        rs.next();
        long nextId = rs.getLong(1);
        rs.close();
        return nextId;
    }

    private void _setNextId() throws SQLException {
        long nextIdDB = _getNextIdFromDB();
        m_nextId = Math.max(m_nextId, nextIdDB);
        pstmtSetNextID.setLong(1, m_nextId);
        pstmtSetNextID.setLong(2, nextIdDB);
        pstmtSetNextID.executeUpdate();
    }

    private long _generateUniqueId() throws SQLException {

        // Get current component ID
        if (m_batchMode) {
            return m_nextId++;
        }
        _setNextId();
        return m_nextId;
    }


    private boolean _storeElement(String element) throws SQLException {
        if (! isElement(element)) {
            pstmtAddElement.setString(1, element);
            if (m_batchMode) {
                pstmtAddElement.addBatch();
                countPstmtAddElement++;
            }
            else
                pstmtAddElement.executeUpdate();

            m_domain.add(element);
            return true;
        }
        return false;
    }


    public String addElement() throws SQLException {
        long id = _generateUniqueId();
        String element = "_" + id;
        _storeElement(element);
        return element;
    }

    public boolean addElement(String name) throws SQLException {
        return _storeElement(name);
    }

    public void removeElement(String id) throws SQLException {
        String sql = "DELETE * FROM Domain WHERE element='" + id + "'";
        executeUpdate(sql);
        m_domain.remove(id);
    }

    public void defineFeature(String feature) throws SQLException {
        if (! isFeature(feature)) {
            pstmtDefineFeature.setString(1, feature);
            pstmtDefineFeature.execute();
            m_feature.add(feature);
        }
    }

    public void addFeature(String element, String feature, String value)
        throws SQLException, FeatureLogicException {

        // Ensure consistency of the data model
        checkElement(element);
        checkUsableAsNonPrimitiveElement(element);
        checkElement(value);
        checkFeature(feature);

        // Add feature pair
        pstmtAddFeature.setString(1, element);
        pstmtAddFeature.setString(2, feature);
        pstmtAddFeature.setString(3, value);
        if (m_batchMode) {
            pstmtAddFeature.addBatch();
            countPstmtAddFeature++;
        }
        else
            pstmtAddFeature.executeUpdate();
    }

    public void changeFeature(String element, String feature, String value)
        throws SQLException, FeatureLogicException {

        // Ensure consistency of the data model
        checkElement(value);

        // Modify feature pair
        pstmtChangeFeature.setString(1, element);
        pstmtChangeFeature.setString(2, feature);
        pstmtChangeFeature.setString(3, value);
        if (m_batchMode) {
            pstmtChangeFeature.addBatch();
            countPstmtChangeFeature++;
        }
        else
            pstmtChangeFeature.executeUpdate();
    }

    public void removeFeature(String element, String feature)
        throws SQLException, FeatureLogicException {
        throw new UnsupportedOperationException();
    }

    public String findOrCreateAtom(int value)
        throws SQLException, FeatureLogicException {
        return _findOrCreateLiteral(String.valueOf(value));
    }

    public String findOrCreateAtom(long value)
        throws SQLException, FeatureLogicException {
        return _findOrCreateLiteral(String.valueOf(value) + "L");
    }

    public String findOrCreateAtom(double value)
        throws SQLException, FeatureLogicException {
        return _findOrCreateLiteral(String.valueOf(value));
    }

    public String findOrCreateAtom(String value)
        throws SQLException, FeatureLogicException {
        return _findOrCreateLiteral("\"" + value + "\"");
    }

    public String findOrCreateAtom(Date date)
        throws SQLException, FeatureLogicException {
        return _findOrCreateLiteral(String.valueOf(date.getTime()) + "DATE");
    }

    private String _findOrCreateLiteral(String literal) throws SQLException {

        // Does the literal already exist?
        String elm = m_atom.getKey(literal);
        if (elm != null)
            return elm;


        // Create a new literal
        String element = addElement();
        pstmtInsertIntoAtom.setString(1, element);
        pstmtInsertIntoAtom.setString(2, literal);
        if (m_batchMode) {
            pstmtInsertIntoAtom.addBatch();
            countPstmtInsertIntoAtom++;
        }
        else
            pstmtInsertIntoAtom.executeUpdate();

        m_atom.put(element, literal);

        // Return the element
        return element;
    }

//  ------------------------------------------------------------------------
//  Miscellaneous
//  ------------------------------------------------------------------------

    public boolean isElement(String element) throws SQLException {
        return m_domain.contains(element);
    }

    public boolean isUsedAsPrimitiveElement(String element)
    throws SQLException {
        return m_atom.containsKey(element);
    }

    public boolean isUsedAsNonPrimitiveElement(String element)
    throws SQLException {
        String sql =
            "SELECT Count(*) AS n FROM RelSlot " +
            "WHERE RelSlot.object='" + element + "'";
        Statement stmt = m_con.createStatement();
        ResultSet rs = stmt.executeQuery(sql);

        rs.next();
        int n = rs.getInt(1);

        rs.close();
        stmt.close();

        return n != 0;
    }

    public boolean isFeature(String feature) throws SQLException {
        return m_feature.contains(feature);
    }

    public String getFeatureValue(String element, String feature)
    throws SQLException, FeatureLogicException {
        checkElement(element);
        checkFeature(feature);

        pstmtGetFeatureValue.setString(1, element);
        pstmtGetFeatureValue.setString(2, feature);
        ResultSet rs = pstmtGetFeatureValue.executeQuery();

        String value = rs.next() ? rs.getString(2) : null;
        rs.close();

        return value;
    }

    public void getFeatureValues(String element, Map<String,String> features)
    throws SQLException, FeatureLogicException {
        checkElement(element);

        pstmtGetFeatureValues.setString(1, element);
        ResultSet rs = pstmtGetFeatureValues.executeQuery();

        features.clear();
        while (rs.next()) {
            features.put(rs.getString(1), rs.getString(2));
        }
        rs.close();
    }

    public Map<String,Map<String,String>> getFeatureValues(Set<String> elements)
    throws SQLException, FeatureLogicException {
        Map<String,Map<String,String>> map = new HashMap<String,Map<String,String>>();
        for (String element : elements) {
            Map<String,String> features = new HashMap<String,String>();
            getFeatureValues(element, features);

            map.put(element, features);
        }
        // Much to slow
        /*String whereIn = null;
        for (String element : elements) {
            if (whereIn == null)
                whereIn = "'" + element + "'";
            else
                whereIn += ",'" + element + "'";
        }
        if (whereIn == null)
            return map;
        String sql = "SELECT * FROM relslot WHERE object IN (" + whereIn +
                         ") ORDER BY object";
        System.out.println(sql);
        Statement stmt = m_con.createStatement();
        ResultSet rs = stmt.executeQuery(sql);
        String element = "";
        Map<String,String> mapFeatures = new HashMap<String,String>();
        while (rs.next()) {
            if (!element.equals(rs.getString(1))) {
                if (element.length() > 0 && mapFeatures.size() > 0)
                    map.put(element, mapFeatures);
                element = rs.getString(1);
                mapFeatures = new HashMap<String,String>();
            }
            mapFeatures.put(rs.getString(2), rs.getString(3));
        }
        if (element.length() > 0 && mapFeatures.size() > 0)
            map.put(element, mapFeatures);
        rs.close();
        */
        return map;
    }

    public Object getAtom(String prmElement)
    throws SQLException, FeatureLogicException {
        checkElement(prmElement);

        /*pstmtGetAtom.setString(1, prmElement);
        ResultSet rs = pstmtGetAtom.executeQuery();

        if (! rs.next())
            throw new
                FeatureLogicException("getAtom(): no primitive element");

        String value = rs.getString(1);
        rs.close();*/

        String value = m_atom.get(prmElement);
        if (value == null)
            throw new FeatureLogicException("getAtom(): no primitive element");

        // Convert string value to typed value
        Object typedValue;
        if (value.startsWith("\""))
            typedValue = value.substring(1, value.length() - 1);
        else if (-1 != value.indexOf('.'))
            typedValue = new Double(value);
        else if (value.endsWith("L")) {
            value = value.substring(0, value.length() - 1);
            typedValue = new Long(value);
        }
        else if (value.endsWith("DATE")) {
            value = value.substring(0, value.length() - 4);
            //typedValue = new Date(new Long(value));
            typedValue = new FormattedDate(new Long(value));
        }
        else
            typedValue = new Integer(value);

        return typedValue;
    }

    public Map<String,Object> getAllAtoms() throws SQLException, FeatureLogicException {
        Map<String,Object> map = new HashMap<String,Object>();
        for (String element : m_atom.keySet()) {
            map.put(element, getAtom(element));
        }
        return map;
    }

//  ------------------------------------------------------------------------
//  Feature Logic algebra
//  ------------------------------------------------------------------------

    public SetDsc top() {
        return new SetDscPostgresql(
            "SELECT element FROM Domain"
        );
    }

    public SetDsc bottom() {
        return new SetDscPostgresql(
            "SELECT element FROM Domain WHERE 0 = 1"
        );
    }

    public SetDsc element(String x) {
        return new SetDscPostgresql(
            "SELECT element FROM Domain WHERE element = '" + x + "'"
        );
    }

    public SetDsc primitiveElement(int atom) {
        return new SetDscPostgresql(
            "SELECT element FROM Atom WHERE value = '" + atom + "'"
        );
    }

    public SetDsc primitiveElement(long atom) {
        return new SetDscPostgresql(
            "SELECT element FROM Atom WHERE value = '" + atom + "'"
        );
    }

    public SetDsc primitiveElement(double atom) {
        return new SetDscPostgresql(
            "SELECT element FROM Atom WHERE value = '" + atom + "'"
        );
    }

    public SetDsc primitiveElement(String atom) {
        return new SetDscPostgresql(
            "SELECT element FROM Atom WHERE value = '\"" + atom + "\"'"
        );
    }

    public SetDsc extract(String f, SetDsc s) {
        return new SetDscPostgresql(
            "SELECT DISTINCT RelSlot.value AS element" +
            " FROM (" + s + ") S, RelSlot" +
            " WHERE RelSlot.feature = '" + f + "' AND" +
            " Relslot.object = S.element"
        );
    }

    public SetDsc selection(String f, SetDsc s) {
        return new SetDscPostgresql(
            //"SELECT DISTINCT Relslot.object AS element" +
            // DISTINCT not necessary here. Costs about 1/3 more time.
            "SELECT Relslot.object AS element" +
            " FROM (" + s + ") S, RelSlot" +
            " WHERE Relslot.feature = '" + f + "' AND" +
            " Relslot.value = S.element"
        );
    }

    public SetDsc existence(String f) {
        return new SetDscPostgresql(
            // "SELECT DISTINCT Relslot.object AS element" +
            // DISTINCT not necessary here. Costs about 1/3 more time.
            "SELECT Relslot.object AS element" +
            " FROM RelSlot WHERE Relslot.feature = '" + f + "'"
        );
    }

    public SetDsc divergence(String f) {
        return new SetDscPostgresql(
            "SELECT element FROM Domain" +
            " EXCEPT" +
            " SELECT DISTINCT object AS element FROM RelSlot" +
            " WHERE feature = '" + f + "'"
        );
    }

    public SetDsc agreement(String f, String g) {
        return new SetDscPostgresql(
            "SELECT DISTINCT A.object AS element" +
            " FROM RelSlot A, RelSlot B" +
            " WHERE" +
            " A.object = B.object AND" +
            " A.feature = '" + f + "' AND" +
            " B.feature = '" + g + "' AND" +
            " A.value = B.value"
        );
    }

    public SetDsc disagreement(String f, String g) {
        return new SetDscPostgresql(
            "SELECT DISTINCT A.object AS element" +
            " FROM RelSlot A, RelSlot B" +
            " WHERE" +
            " A.object = B.object AND" +
            " A.feature = '" + f + "' AND" +
            " B.feature = '" + g + "' AND" +
            " A.value <> B.value"
        );
    }

    public SetDsc union(SetDsc s, SetDsc t) {
        return new SetDscPostgresql(
            "(" + s + ") UNION (" + t + ")"
        );
    }

    public SetDsc intersection(SetDsc s, SetDsc t) {
        return new SetDscPostgresql(
            "(" + s + ") INTERSECT (" + t + ")"
        );
    }

    public SetDsc difference(SetDsc s, SetDsc t) {
        return new SetDscPostgresql(
            "(" + s + ") EXCEPT (" + t + ")"
        );
    }

    public SetDsc complement(SetDsc s) {
        return new SetDscPostgresql(
            "SELECT element FROM Domain EXCEPT (" + s + ")"
        );
    }

    public SetDsc implication(SetDsc s, SetDsc t) {
       return union(complement(s), t);
    }

    public SetDsc equivalence(SetDsc s, SetDsc t) {
        return intersection(implication(s, t), implication(t, s));
    }

    public int cardinality(SetDsc s)
    throws SQLException, FeatureLogicException {

        String sql = "SELECT Count(*) AS n FROM (" + s + ")";

        Statement stmt = m_con.createStatement();
        ResultSet rs = stmt.executeQuery(sql);

        int n = 0;
        if (rs.next())
            n = rs.getInt(1);

        rs.close();
        stmt.close();

        return n;
    }

//  ------------------------------------------------------------------------
//  Iteration of the result set
//  ------------------------------------------------------------------------

    public ResultSetIterator resultSetIterator(SetDsc s) throws SQLException {
        Statement statement = m_con.createStatement();
        ResultSet resultSet = statement.executeQuery(
            "SELECT element FROM Domain WHERE 0 = 1 UNION (" + s.toString() + ")"
        );
        return new ResultSetIteratorImp(statement, resultSet);
    }

//  ------------------------------------------------------------------------
//  Helpers
//  ------------------------------------------------------------------------

   /**
    * Executes a SQL Update statement
    * @param sql SQL string
    */
    private void executeUpdate(String sql) throws SQLException {
        Statement stmt = m_con.createStatement();
        stmt.executeUpdate(sql);
        stmt.close();
    }

   /**
    * Checks whether an element exists in the Domain. On error an exception
    * is thrown.
    * @param element the element to be checked
    */
    private void checkElement(String element)
        throws SQLException, FeatureLogicException {

        if (! isElement(element))
            throw new FeatureLogicException("Invalid element " + element);
    }

   /**
    * Checks whether an element is usable as non-primitive element. If the
    * element is already used as primitive an exception is thrown.
    * @param element the element to be checked
    */
    private void checkUsableAsNonPrimitiveElement(String element)
        throws SQLException, FeatureLogicException {

        if (isUsedAsPrimitiveElement(element))
            throw new FeatureLogicException(
                "Attempt to use the primitive element " + element +
                " as non-primitive element");
    }

   /**
    * Checks whether an element is usable as non-primitive element. If the
    * element is already used as primitive an exception is thrown.
    * @param element the element to be checked
    */
    private void checkUsableAsPrimitiveElement(String element)
        throws SQLException, FeatureLogicException {

        if (isUsedAsNonPrimitiveElement(element))
            throw new FeatureLogicException(
                "Attempt to use the non-primitive element " + element +
                " as primitive element");
    }

   /**
    * Checks whether a feature is defined. On error an exception is thrown.
    * @param feature the feature name to be checked
    */
    private void checkFeature(String feature)
        throws SQLException, FeatureLogicException {

        if (! isFeature(feature))
            throw new FeatureLogicException("Invalid feature " + feature);
    }

    public String toString() {
        return "Domain: " + m_domain.size() + " elements. Atoms: " + m_atom.size();
    }
}