/*
 * Copyright (C) 2002-2010 Berthold Firmenich
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.update;

import java.util.Set;

/**
 * This interface provides definitions for objects that represent an updatable,
 * directed graph.
 *
 * @author Christian Koch
 * @version $Revision: 1.8 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 * @see cib.util.update.Graph
 * @see cib.util.update.Update
 * @see cib.util.update.Node
 */
@Deprecated
public interface UpdatableGraph extends Graph {

    /**
     * Gets the link graph as matrix.
     * @return the link graph's matrix
     */
    public boolean[][] getLink();

    /**
     * Gets the hull of this link graph as matrix.
     * @return the hull matrix
     */
    public boolean[][] getHull();

    /**
     * Gets the link graph.
     * @return the link graph
     */
    public Graph getLinkGraph();

    /**
     * Gets the set of changed nodes.
     * @return the changed nodes as set
     */
    public Set<Node> getChangedNodeSet();

    /**
     * Gets the set of nodes in the link graph.
     * @return the set of link graph's nodes.
     */
    public Set<Node> getNodeSet();

    /**
     * Sets the update object for this upatable graph.
     * @param update the update object
     */
    public void setUpdate(Update update);

    /**
     * Starts the update associated with this updatable graph.
     */
    public void update(Object context);

    /**
     * Adds a specified node to the set of changed nodes.
     */
    public void wasChanged(Node node);

    /**
     * Updates a specified node.
     * @param node the specified node
     * @param context an application specific context
     */
    public void updateNode(Node node, Object context);

}//end of class