/* 
 * Copyright (C) 2002-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.update;

import java.util.Set;

/** 
 * This interface provides definitions for objects that represent an update.
 *
 * @author Christian Koch
 * @version $Revision: 1.11 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 * @see cib.util.update.Graph
 * @see cib.util.update.Update
 * @see cib.util.update.Node
 */
@Deprecated
public interface Update {
    
    /** 
     * Sets the updatable graph object for this update.
     * @param ug the updatable graph object
     */   
    public void setUpdatableGraph(UpdatableGraph ug);
    
    /** 
     * Adds a specified node to the set of destination nodes of this update.
     * @param node the specified node
     * @return <code>true</code> if the node was added
     */
    public boolean addDestNode(Node node);
    
    /** 
     * Removes a specified node from the set of destination nodes of this update.
     * @param node the specified node
     * @return <code>true</code> if the node was removed
     */
    public boolean removeDestNode(Node node);
    
    /** 
     * Determines the node sequence in the update process and updates nodes
     * sequentially.
     * @param context an application specific context
     */
    public void update(Object context);
    
    /** 
     * Gets the set of domain nodes of this update.
     * @return the set of domain nodes
     */
    public Set getDomainSet();
    
    /** 
     * Gets the set of destination nodes of this update.
     * @return the set of destination nodes
     */
    public Set getDestSet();
    
    /** 
     * Determines if a specified node has still to be updated.
     * @param n the specified node
     * @return <code>true</code> if the node has still to be updated
     */
    public boolean stillToUpdate(Node n);

    /** 
     * Determines if a specified node has already been updated.
     * @param n the specified node
     * @return <code>true</code> if the node has already been updated
     */
    public boolean alreadyUpdated(Node n);
    
}//end of class