/* 
 * Copyright (C) 2002-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.update;

import java.util.Iterator;

/** 
 * This interface provides definitions for objects that represent an update
 * interface between objects and graph nodes.
 *
 * @author Christian Koch
 * @version $Revision: 1.8 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 * @see cib.util.update.Node
 * @see cib.util.update.Update
 * @see cib.util.update.GraphException
 */
@Deprecated
public interface UpdateInterface {
    
    /** 
     * Creates a node for a specified updatable object and adds it to the
     * updatable link graph.
     * @param u the specified updatable object
     * @return <code>true</code> if the node was added
     */
    public boolean addNode(Updatable u);
    
    /** 
     * Removes a node for a specified updatable object from the updatable link graph.
     * @param u the specified updatable object
     * @return <code>true</code> if the node was removed
     */
    public boolean removeNode(Updatable u);
    
    /** 
     * Adds an edge to the updatable link graph between source node and
     * destination node for specified source and destination objects.
     * @param src the specified source object
     * @param dest the specified destination object
     * @return <code>true</code> if the edge was added
     * @throws GraphException a graph exception
     */
    public boolean addEdge(Updatable src, Updatable dest) throws GraphException;
    
    /** 
     * Removes an edge from the updatable link graph between source node and
     * destination node for specified source and destination objects.
     * @param src the specified source object
     * @param dest the specified destination object
     * @return <code>true</code> if the edge was removed
     */
    public boolean removeEdge(Updatable src, Updatable dest);
    
    /** 
     * Starts an update concerning all node objects as destination objects.
     * @param context an application specific context
     */
    public void update(Object context);
    
    /** 
     * Starts an update concerning the objects described by the specified iterator
     * object. 
     * @param it the iterator that describes the destination objects
     * @param context an application specific context
     */
    public void update(Iterator it, Object context);
    
    /** 
     * Adds a node for a specified updatable object to the set of changed nodes in the
     * updatable link graph.
     * @param u the specified updatable object
     */
    public void wasChanged(Updatable u);
    
    /** 
     * Gets the updatable object for a specified node object.
     * @param node the specified node object
     * @return the corrensponding updatable object
     */
    public Updatable getUpdatable(Node node);
    
    /** 
     * Gets the node object for a specified updatable object.
     * @param u the specified updatable object
     * @return the corrensponding node object
     */
    public Node getNode(Updatable u);
}//end of class