/* 
 * Copyright (C) 2002-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.view;

import java.awt.Color;
import java.awt.Paint;
import java.awt.geom.AffineTransform;
import java.io.Serializable;

/** 
 * This is the interface for a scalable and markable paint.
 * 
 * @author Christian Koch
 * @version $Revision: 1.5 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public interface FillPaint extends Paint, Serializable, Cloneable {
    /**
     * Sets the factor f to convert from paper to model units:
     * <p> 
     *     f = scale / mm_per_natural_unit
     * <p>
     * If the natural unit is the [m] and the scale is 1:100 then
     * f = 100 / 1000 = .1
     * 
     * @param p2m the scale factor
     */
    public void setPaperToModel(double p2m);
    /**
     * Returns the scale factor to convert from paper to model units. 
     * 
     * @return the scale factor
     */
    public double getPaperToModel();
    /**
     * Sets the model to view transform.
     * 
     * @param m2v the model to view transform
     */
    public void setModelToView(AffineTransform m2v);
    /**
     * Gets the model to view transform.
     * 
     * @return the model to view transform
     */
    public AffineTransform getModelToView();
    /**
     * Sets the fill pattern marked.
     * 
     * @param marked
     */
    public void setMarked(boolean marked);
    /**
     * Sets the transparency.
     * 
     * @param alpha The alpha component in the range 0 - 255.
     */
    public void setTransparency(int alpha);
    /**
     * Clones the FillPaint.
     * 
     * @return cloned object
     */
    public Object clone();
    /**
     * Calculates and returns a replacement color or null. This is necessary for
     * platforms where the resource usage gets very high if the user is zoomed
     * in deeply.  
     * 
     * @return null if painting with this Paint should be performed, a
     * replacement color otherwise
     */
    public Color getReplacementColor();
}