/* 
 * Copyright (C) 2002-2010 Berthold Firmenich
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, look at "http://www.gnu.org/copyleft/gpl.html" or
 * write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 * 
 * Contact address: berthold.firmenich@cademia.org
 * Homepage       : http://www.cademia.org/
 */

package cib.util.view;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Insets;

/** 
 * FlowLayout for the toolbar panel including line wrapping.
 * 
 * @author Christian Koch
 * @version $Revision: 1.5 $, $Author: cvs-cad-firmenic $
 * @version $Date: 2010/01/13 15:05:11 $
 */
public class ToolBarLayout extends FlowLayout {

    private static final long serialVersionUID = 1L;
    
    // override
    public Dimension preferredLayoutSize(Container target) {
        synchronized (target.getTreeLock()) {
            Dimension dim = new Dimension(0, 0);
            Insets insets = target.getInsets();

            int n = target.getComponentCount();
            int rows = 1;
            int width = insets.left + insets.right;
            int maxWidth = target.getSize().width;
            int height = 0;
            int r = 0;

            // iterate over all comps
            for (int i = 0 ; i < n ; i++) {
                Component c = target.getComponent(i);
                // check if visible
                if (c.isVisible()) {
                    Dimension d = c.getPreferredSize();
                    // max height
                    height = Math.max(height, d.height);    
                    // add hor. gap
                    if (i > 0) {
                        width += getHgap();
                    }
                    // add comp. width
                    width += d.width;
                    // check overflow
                    if ((width + getHgap()) > maxWidth) {
                        // if any comps in row
                        if (r > 0) {
                            rows++;
                            width = insets.left + insets.right + d.width;
                            r = 0;
                        }
                    }
                    r++;
                }
            }
            // set target width
            dim.width = maxWidth;
            // set target height
            dim.height = insets.top + insets.bottom + rows*height +
                getVgap()*(rows - 1);
            return dim;
        }
    }
}